@@ -95,6 +95,8 @@
 
 #undef HAVE_H_ERRNO_DECL
 
+#undef HAVE_UNSETENV_DECL
+
 #undef NEED_YYWRAP
 
 #undef HAVE_POLL
@@ -121,9 +123,6 @@
 
 #undef HAVE_INET_ATON_DECL
 
-/* Define to `int' if <sys/socket.h> doesn't define.  */
-#undef socklen_t
-
 /* define if you are building with SOCKS support */
 #undef SOCKS4
 
@@ -164,6 +163,10 @@
 CDECL const char *strerror(int);
 #endif
 
+#ifdef HAVE_LOCALE_H
+# include <locale.h>
+#endif
+
 #ifndef HAVE_SETLOCALE
 # define setlocale(Category, Locale)
 #endif
@@ -250,13 +253,6 @@ CDECL void SOCKSinit(const char *);
 # define __builtin_expect(expr, val) (expr)
 #endif
 
-#if defined(__cplusplus) && !defined(HAVE_CXX_BOOL)
-typedef unsigned _bool;
-#define bool   _bool
-#define false  0U
-#define true   1U
-#endif
-
 #ifdef __GNUC__
 # define PRINTF_LIKE(n,m) __attribute__((format(__printf__,n,m)))
 #else
