@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: CatJob.h,v 1.7 2000/07/19 07:24:14 lav Exp $ */
+/* $Id: CatJob.h,v 1.9 2002/01/10 15:30:58 lav Exp $ */
 
 #ifndef CATJOB_H
 #define CATJOB_H
@@ -32,9 +32,10 @@ class CatJob : public CopyJobEnv
 {
 protected:
    FDStream *global;
-   const char *for_each;
+   ArgV *for_each;
    bool ascii;
    bool auto_ascii;
+   bool fail_if_broken;
 
    void	NextFile();
 
