@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-1999 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2002 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: CmdExec.h,v 1.42 2001/07/26 09:57:18 lav Exp $ */
+/* $Id: CmdExec.h,v 1.45 2002/01/04 12:31:29 lav Exp $ */
 
 #ifndef CMDEXEC_H
 #define CMDEXEC_H
@@ -31,8 +31,10 @@
 #include "alias.h"
 #include "history.h"
 #include "bookmark.h"
+#include "Glob.h"
 
 class QueueFeeder;
+class LocalDirectory;
 
 extern History cwd_history;
 extern Bookmark lftp_bookmarks;
@@ -144,6 +146,7 @@ public:
    void PrependCmd(const char *c);
    void ExecParsed(ArgV *a,FDStream *o=0,bool b=false);
    static void unquote(char *buf,const char *str);
+   static char *unquote(const char *str);
    static bool needs_quotation(const char *buf);
    void FeedQuoted(const char *c);
    void AtExit();
@@ -153,7 +156,7 @@ public:
 
    void SuspendJob(Job *j);
 
-   CmdExec(FileAccess *s);
+   CmdExec(FileAccess *s,LocalDirectory *c);
    ~CmdExec();
 
    bool Idle();	// when we have no command running and command buffer is empty
@@ -190,10 +193,9 @@ public:
    time_t start_time;
 
    static CmdExec *cwd_owner;
-   char	 *cwd;
+   LocalDirectory *cwd;
    void	 SaveCWD();
    int	 RestoreCWD();
-   void  SetCWD(const char *c);
 
    FDStream *default_output;
 
