@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileAccess.cc,v 1.87 2001/10/09 12:26:04 lav Exp $ */
+/* $Id: FileAccess.cc,v 1.94 2001/12/20 10:24:55 lav Exp $ */
 
 #include <config.h>
 
@@ -33,10 +33,6 @@
 #include <errno.h>
 #include "ascii_ctype.h"
 #include <fcntl.h>
-/* Our autoconf test will switch to lib/fnmatch.c if the local fnmatch
- * isn't GNU, so this should be OK. */
-#define _GNU_SOURCE
-#include <fnmatch.h>
 #include "LsCache.h"
 #include "xalloca.h"
 #include "log.h"
@@ -92,7 +88,7 @@ void FileAccess::Init()
 
    chmod_mode=0644;
 
-   priority=0;
+   priority=last_priority=0;
 
    next=chain;
    chain=this;
@@ -251,7 +247,7 @@ void FileAccess::CloseOnExec(int fd)
 void  FileAccess::Open(const char *fn,int mode,off_t offs)
 {
 #ifdef OPEN_DEBUG
-   printf("FA::Open(%s)\n",fn);
+   printf("FA::Open(%s,%d)\n",fn?fn:"NULL",mode);
 #endif
    if(IsOpen())
       Close();
@@ -560,6 +556,8 @@ void  FileAccess::ExpandTildeInCWD()
 	 expand_tilde(&file,home);
       if(file1)
 	 expand_tilde(&file1,home);
+      if(mode==CHANGE_DIR)
+	 OptimizePath(file);
    }
 }
 
@@ -596,67 +594,35 @@ int FileAccess::device_prefix_len(const char *path)
    return 0;
 }
 
-void FileAccess::Chdir(const char *path,bool verify)
+void FileAccess::OptimizePath(char *path)
 {
-   char	 *newcwd=(char*)alloca(strlen(cwd)+strlen(path)+2);
    int	 prefix_size=0;
 
-   ExpandTildeInCWD();
-
-   if(cwd[0]=='/')
+   if(path[0]=='/')
    {
       prefix_size=1;
-      if(cwd[1]=='/' && cwd[2] && cwd[2]!='/')
+      if(path[1]=='/' && (!path[2] || path[2]!='/'))
 	 prefix_size=2;
    }
-   else if(cwd[0]=='~')
+   else if(path[0]=='~')
    {
       prefix_size=1;
-      while(cwd[prefix_size]!='/' && cwd[prefix_size]!='\0')
+      while(path[prefix_size]!='/' && path[prefix_size]!='\0')
 	 prefix_size++;
    }
    else
    {
       // handle VMS and DOS devices.
-      prefix_size=device_prefix_len(cwd);
-   }
-
-   int dev_prefix=0;
-
-   if(path[0]=='/')
-   {
-      strcpy(newcwd,path);
-      prefix_size=1;
-      if(path[1]=='/' && path[2] && path[2]!='/')
-	 prefix_size=2;
-   }
-   else if(path[0]=='~')
-   {
-      strcpy(newcwd,path);
-      prefix_size=1;
-      while(newcwd[prefix_size]!='/' && newcwd[prefix_size]!='\0')
-	 prefix_size++;
-   }
-   else if((dev_prefix=device_prefix_len(path))>0)
-   {
-      strcpy(newcwd,path);
-      prefix_size=dev_prefix;
-   }
-   else
-   {
-      if(cwd[0])
-	 sprintf(newcwd,"%s/%s",cwd,path);
-      else
-	 strcpy(newcwd,path);
+      prefix_size=device_prefix_len(path);
    }
 
    char	 *in;
    char	 *out;
 
-   in=out=newcwd+prefix_size;
+   in=out=path+prefix_size;
 
    while((in[0]=='.' && (in[1]=='/' || in[1]==0))
-   || (in>newcwd && in[-1]=='/' && (in[0]=='/'
+   || (in>path && in[-1]=='/' && (in[0]=='/'
 	 || (in[0]=='.' && in[1]=='.' && (in[2]=='/' || in[2]==0)))))
    {
       if(in[0]=='.' && in[1]=='.')
@@ -687,18 +653,18 @@ void FileAccess::Chdir(const char *path,bool verify)
 	    // .. - prev dir
 	    if(in[2]=='.' && (in[3]=='/' || in[3]=='\0'))
 	    {
-	       if(last_element_is_doubledot(newcwd+prefix_size,out)
-	       || out==newcwd
-	       || (out==newcwd+prefix_size && out[-1]!='/'))
+	       if(last_element_is_doubledot(path+prefix_size,out)
+	       || out==path
+	       || (out==path+prefix_size && out[-1]!='/'))
 	       {
-		  if(out>newcwd && out[-1]!='/')
+		  if(out>path && out[-1]!='/')
 		     *out++='/';
 		  *out++='.';
 		  *out++='.';
 	       }
 	       else
 	       {
-		  while(out>newcwd+prefix_size && *--out!='/')
+		  while(out>path+prefix_size && *--out!='/')
 		     ;
 	       }
 	       in+=3;
@@ -706,7 +672,7 @@ void FileAccess::Chdir(const char *path,bool verify)
 	    }
 	 }
 	 // don't add slash after prefix with slash
-	 if(out>newcwd && out[-1]=='/')
+	 if(out>path && out[-1]=='/')
 	 {
 	    in++;
 	    continue;
@@ -715,13 +681,39 @@ void FileAccess::Chdir(const char *path,bool verify)
       if((*out++=*in++)=='\0')
 	 break;
    }
+}
+
+void FileAccess::Chdir(const char *path,bool verify)
+{
+   ExpandTildeInCWD();
+
+   char	 *newcwd=(char*)alloca(strlen(cwd)+strlen(path)+2);
+
+   if(path[0]=='/')
+      strcpy(newcwd,path);
+   else if(path[0]=='~')
+      strcpy(newcwd,path);
+   else if(device_prefix_len(path)>0)
+      strcpy(newcwd,path);
+   else
+   {
+      if(cwd[0])
+      {
+	 if(cwd[strlen(cwd)-1]=='/')
+	    sprintf(newcwd,"%s%s",cwd,path);
+	 else
+	    sprintf(newcwd,"%s/%s",cwd,path);
+      }
+      else
+	 strcpy(newcwd,path);
+   }
+
+   OptimizePath(newcwd);
+
    if(verify)
       Open(newcwd,CHANGE_DIR);
    else
-   {
-      xfree(cwd);
-      cwd=xstrdup(newcwd);
-   }
+      SetCwd(newcwd);
 }
 
 void FileAccess::Chmod(const char *file,int m)
@@ -911,7 +903,7 @@ void FileAccess::UseCache(bool) {}
 bool FileAccess::NeedSizeDateBeforehand() { return false; }
 void FileAccess::Cleanup() {}
 void FileAccess::CleanupThis() {}
-ListInfo *FileAccess::MakeListInfo() { return 0; }
+ListInfo *FileAccess::MakeListInfo(const char *path) { return 0; }
 Glob *FileAccess::MakeGlob(const char *pattern) { return 0; }
 DirList *FileAccess::MakeDirList(ArgV *a) { if(a) delete a; return 0; }
 
@@ -1042,217 +1034,40 @@ void FileAccessOperation::SetError(const char *e)
 
 
 // ListInfo implementation
-ListInfo::ListInfo()
+ListInfo::ListInfo(FileAccess *s,const char *p)
 {
+   session=s;
+   saved_cwd=0;
    result=0;
 
-   rxc_exclude=0;
-   rxc_include=0;
-   path=0;
+   exclude=0;
+   exclude_prefix=0;
 
    need=0;
 
    follow_symlinks=false;
+
+   if(session && p)
+   {
+      saved_cwd=xstrdup(session->GetCwd());
+      session->Chdir(p,false);
+   }
 }
 
 ListInfo::~ListInfo()
 {
-   if(result)
-      delete result;
+   if(session)
+      session->Close();
+   if(session && saved_cwd)
+      session->SetCwd(saved_cwd);
+   delete result;
+   xfree(saved_cwd);
 }
 
-void ListInfo::SetExclude(const char *p,regex_t *x,regex_t *i)
+void ListInfo::SetExclude(const char *p,PatternSet *x)
 {
-   rxc_exclude=x;
-   rxc_include=i;
-   path=p;
-}
-
-// Glob implementation
-Glob::~Glob()
-{
-   xfree(pattern);
-}
-
-Glob::Glob(const char *p)
-{
-   pattern=xstrdup(p);
-   dirs_only=false;
-   files_only=false;
-   match_period=true;
-   inhibit_tilde=true;
-   casefold=false;
-
-   if(pattern[0]=='~')
-   {
-      char *slash=strchr(pattern,'/');
-      if(slash)
-      {
-	 *slash=0;
-	 inhibit_tilde=HasWildcards(pattern);
-	 *slash='/';
-      }
-      else
-	 inhibit_tilde=HasWildcards(pattern);;
-   }
-   if(pattern[0] && !HasWildcards(pattern))
-   {
-      // no need to glob, just unquote
-      char *u=alloca_strdup(pattern);
-      UnquoteWildcards(u);
-      add(new FileInfo(u));
-      done=true;
-   }
-}
-
-void Glob::add_force(const FileInfo *info)
-{
-   // insert new file name into list
-   list.Add(new FileInfo(*info));
-}
-void Glob::add(const FileInfo *info)
-{
-   if(info->defined&info->TYPE)
-   {
-      if(dirs_only && info->filetype==info->NORMAL)
-	 return;   // note that symlinks can point to directories,
-		   // so skip normal files only.
-      if(files_only && info->filetype==info->DIRECTORY)
-	 return;
-   }
-
-   char *s=info->name;
-   if(s==0)
-      return;
-
-   int flags=FNM_PATHNAME;
-   if(match_period)
-      flags|=FNM_PERIOD;
-
-   if(casefold)
-      flags|=FNM_CASEFOLD;
-
-   if(pattern[0]!=0
-   && fnmatch(pattern, s, flags)!=0)
-      return; // unmatched
-
-   if(s[0]=='~' && inhibit_tilde)
-   {
-      char *new_name=alloca_strdup2(s,2);
-      strcpy(new_name,"./");
-      strcat(new_name,s);
-      FileInfo new_info(*info);
-      new_info.SetName(new_name);
-      add_force(&new_info);
-   }
-   else
-   {
-      add_force(info);
-   }
-}
-
-bool Glob::HasWildcards(const char *s)
-{
-   while(*s)
-   {
-      switch(*s)
-      {
-      case '\\':
-	 if(s[1])
-	    s++;
-	 break;
-      case '*':
-      case '[':
-      case ']':
-      case '?':
-	 return true;
-      }
-      s++;
-   }
-   return false;
-}
-
-void Glob::UnquoteWildcards(char *s)
-{
-   char *store=s;
-   for(;;)
-   {
-      if(*s=='\\')
-      {
-	 if(s[1]=='*'
-	 || s[1]=='['
-	 || s[1]==']'
-	 || s[1]=='?'
-	 || s[1]=='\\')
-	    s++;
-      }
-      *store=*s;
-      if(*s==0)
-	 break;
-      s++;
-      store++;
-   }
-}
-
-int NoGlob::Do()
-{
-   if(!done)
-   {
-      if(!HasWildcards(pattern))
-      {
-	 char *p=alloca_strdup(pattern);
-	 UnquoteWildcards(p);
-	 add(new FileInfo(p));
-      }
-      done=true;
-      return MOVED;
-   }
-   return STALL;
-}
-NoGlob::NoGlob(const char *p) : Glob(p)
-{
-}
-
-GlobURL::GlobURL(FileAccess *s,const char *p)
-{
-   session=s;
-   reuse=false;
-   glob=0;
-   url_prefix=xstrdup(p);
-   url_prefix[url::path_index(p)]=0;
-
-   ParsedURL p_url(p,true);
-   if(p_url.proto && p_url.path)
-   {
-      session=FA::New(&p_url);
-      if(session)
-      {
-	 glob=session->MakeGlob(p_url.path);
-	 reuse=true;
-      }
-   }
-   else
-   {
-      glob=session->MakeGlob(p);
-   }
-   if(!glob)
-      glob=new NoGlob(p);
-}
-GlobURL::~GlobURL()
-{
-   SMTask::Delete(glob);
-   if(session && reuse)
-      SessionPool::Reuse(session);
-   xfree(url_prefix);
-}
-FileSet *GlobURL::GetResult()
-{
-   FileSet &list=*glob->GetResult();
-   if(!reuse)
-      return &list;
-   for(int i=0; list[i]; i++)
-      list[i]->SetName(url_file(url_prefix,list[i]->name));
-   return &list;
+   exclude=x;
+   exclude_prefix=p;
 }
 
 #include "modconfig.h"
