@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileCopy.cc,v 1.76 2001/11/06 13:15:32 lav Exp $ */
+/* $Id: FileCopy.cc,v 1.83 2002/01/10 09:52:25 lav Exp $ */
 
 /* FileCopyPeer behaviour:
     1) when suspended, does nothing
@@ -72,6 +72,9 @@ int FileCopy::Do()
       state=INITIAL;
       m=MOVED;
    case(INITIAL):
+      if(remove_target_first && !put->FileRemoved())
+	 return m;
+      remove_target_first=false;
       if(put->NeedSizeDateBeforehand())
       {
 	 if(get->GetSize()==NO_SIZE_YET || get->GetDate()==NO_DATE_YET)
@@ -142,6 +145,11 @@ int FileCopy::Do()
       if(put->Broken())
       {
 	 debug((9,_("copy: put is broken\n")));
+	 if(fail_if_broken)
+	 {
+	    SetError(strerror(EPIPE));
+	    return MOVED;
+	 }
 	 goto pre_GET_DONE_WAIT;
       }
       put->Resume();
@@ -371,7 +379,9 @@ void FileCopy::Init()
    put_eof_pos=0;
    bytes_count=0;
    fail_if_cannot_seek=false;
+   fail_if_broken=true;
    remove_source_later=false;
+   remove_target_first=false;
    line_buffer=0;
    line_buffer_max=0;
 }
@@ -619,10 +629,7 @@ bool FileCopyPeer::Done()
       if(removing)
 	 return false;
       if(mode==PUT)
-      {
-	 if(do_set_date && !date_set)
-	    return false;
-      }
+	 return done;
       return true;
    }
    return false;
@@ -637,16 +644,18 @@ FileCopyPeer::FileCopyPeer(direction m)
    e_size=NO_SIZE;
    date=NO_DATE_YET;
    seek_pos=0;
-   can_seek=true;
-   can_seek0=true;
+   can_seek=false;
+   can_seek0=false;
    date_set=false;
    do_set_date=true;
    ascii=false;
    range_start=0;
    range_limit=FILE_END;
    removing=false;
+   file_removed=false;
    use_cache=true;
    write_allowed=true;
+   done=false;
    Suspend();  // don't do anything too early
 }
 FileCopyPeer::~FileCopyPeer()
@@ -667,8 +676,10 @@ int FileCopyPeerFA::Do()
       if(res<=0)
       {
 	 removing=false;
+	 file_removed=true;
 	 session->Close();
-	 m=MOVED;
+	 Suspend();
+	 return MOVED;
       }
       else
 	 return m;
@@ -730,6 +741,7 @@ int FileCopyPeerFA::Do()
 	    fxp_eof:
 	       // FIXME: set date for real.
 	       date_set=true;
+	       done=true;
 	       m=MOVED;
 	    }
 	    else if(res==FA::IN_PROGRESS)
@@ -1077,6 +1089,8 @@ void FileCopyPeerFA::Init()
    try_time=0;
    retries=0;
    redirections=0;
+   can_seek=true;
+   can_seek0=true;
 }
 
 FileCopyPeerFA::FileCopyPeerFA(FileAccess *s,const char *f,int m)
@@ -1228,6 +1242,7 @@ int FileCopyPeerFDStream::getfd()
       }
       return -1;
    }
+   stream->clear_status();
    pos=0;
    if(mode==PUT)
       pos+=in_buffer;
@@ -1246,15 +1261,19 @@ int FileCopyPeerFDStream::Do()
       {
 	 if(eof)
 	 {
-	    if(date_set || date==NO_DATE_YET)
-	       return m;
-	    if(date!=NO_DATE && do_set_date)
+	    if(!date_set && date!=NO_DATE && do_set_date)
 	    {
+	       if(date==NO_DATE_YET)
+		  return m;
 	       if(getfd()==-1)
 		  return m;
 	       stream->setmtime(date);
+	       date_set=true;
+	       m=MOVED;
 	    }
-	    date_set=true;
+	    if(stream && delete_stream && !stream->Done())
+	       return m;
+	    done=true;
 	    return MOVED;
 	 }
 	 if(seek_pos==0)
@@ -1307,15 +1326,6 @@ bool FileCopyPeerFDStream::IOReady()
    return seek_pos==pos || stream->fd!=-1;
 }
 
-bool FileCopyPeerFDStream::Done()
-{
-   if(!super::Done())
-      return false;
-   if(stream && delete_stream && !stream->Done())
-      return false;
-   return true;
-}
-
 void FileCopyPeerFDStream::Seek(off_t new_pos)
 {
    if(pos==new_pos)
@@ -1401,10 +1411,13 @@ int FileCopyPeerFDStream::Get_LL(int len)
 	 Block(fd,POLLIN);
 	 return 0;
       }
+      if(stream->NonFatalError(errno))
+	 return 0;
       stream->MakeErrorText();
       SetError(stream->error_text);
       return -1;
    }
+   stream->clear_status();
 
 #ifndef NATIVE_CRLF
    if(ascii)
@@ -1486,10 +1499,13 @@ int FileCopyPeerFDStream::Put_LL(const char *buf,int len)
 	 broken=true;
 	 return -1;
       }
+      if(stream->NonFatalError(errno))
+	 return 0;
       stream->MakeErrorText();
       SetError(stream->error_text);
       return -1;
    }
+   stream->clear_status();
    if(res==len)
       res+=skip_cr;
    return res;
@@ -1507,8 +1523,21 @@ void FileCopyPeerFDStream::RemoveFile()
 {
    stream->remove();
    removing=false;   // it is instant.
+   file_removed=true;
+   Suspend();
+   current->Timeout(0);
 }
 
+const char *FileCopyPeerFDStream::GetStatus()
+{
+   return stream->status;
+}
+void FileCopyPeerFDStream::Kill(int sig)
+{
+   stream->Kill(sig);
+}
+
+
 FileCopyPeerFDStream *FileCopyPeerFDStream::NewPut(const char *file,bool cont)
 {
    return new FileCopyPeerFDStream(new FileStream(file,O_WRONLY|O_CREAT
@@ -1532,6 +1561,8 @@ FileCopyPeerString::FileCopyPeerString(const char *s, int len)
    Put(s,len);
    eof=true;
    pos=0;
+   can_seek=true;
+   can_seek0=true;
 }
 FileCopyPeerString::~FileCopyPeerString()
 {
