@@ -47,16 +47,18 @@ FDStream::FDStream(int new_fd,const char *new_name)
    else
       name=0;
    error_text=0;
+   status=0;
 }
 FDStream::FDStream()
 {
    fd=-1;
    name=0;
    error_text=0;
+   status=0;
 }
 void FDStream::MakeErrorText()
 {
-   if(errno==ENFILE || errno==EMFILE || E_RETRY(errno))
+   if(NonFatalError(errno))
       return;  // not a serious error - can be retried
    char *syserr=strerror(errno);
    error_text=(char*)xmalloc(strlen(name)+strlen(syserr)+3);
@@ -132,7 +134,7 @@ int OutputFilter::getfd()
       if(second_fd==-1)
       {
 	 if(second->error())
-	    error_text=xstrdup(_("chain output error"));
+	    error_text=xstrdup(second->error_text);
 	 return -1;
       }
       if(pg==0)
@@ -145,7 +147,7 @@ int OutputFilter::getfd()
 
    if(pipe(p)==-1)
    {
-      if(errno==EMFILE || errno==ENFILE)
+      if(NonFatalError(errno))
 	 return -1;
       sprintf(s,_("pipe() failed: %s"),strerror(errno));
       error_text=xstrdup(s);
@@ -154,6 +156,8 @@ int OutputFilter::getfd()
 
    ProcWait::Signal(false);
 
+   bool had_pg=(pg!=0);
+
    fflush(stderr);
    switch(pid=fork())
    {
@@ -184,21 +188,27 @@ int OutputFilter::getfd()
       close(p[1]);
       goto out;
    }
-   /* parent */
-   Parent(p);
 
    if(pg==0)
       pg=pid;
 
+   /* parent */
+   Parent(p);
+
    fcntl(fd,F_SETFD,FD_CLOEXEC);
    fcntl(fd,F_SETFL,O_NONBLOCK);
 
    xfree(oldcwd);
    oldcwd=0;
 
+   // wait until the child stops.
    int info;
    waitpid(pid,&info,WUNTRACED);
+
    w=new ProcWait(pid);
+
+   if(had_pg)
+      kill(pid,SIGCONT);
 out:
    ProcWait::Signal(true);
    return fd;
@@ -214,6 +224,7 @@ void OutputFilter::Init()
    pg=0;
    closed=false;
    stderr_to_stdout=false;
+   delete_second=false;
 }
 
 void OutputFilter::SetCwd(const char *cwd)
@@ -264,6 +275,9 @@ OutputFilter::~OutputFilter()
       w->Auto();
 
    xfree(oldcwd);
+
+   if(delete_second)
+      delete second;
 }
 
 bool OutputFilter::Done()
@@ -277,7 +291,11 @@ bool OutputFilter::Done()
       closed=true;
    }
    if(w->GetState()!=w->RUNNING)
+   {
+      if(delete_second && second)
+	 return second->Done();
       return true;
+   }
    return false;
 }
 bool OutputFilter::broken()
@@ -290,6 +308,23 @@ bool OutputFilter::broken()
       return true; // filter process terminated - pipe is broken
    return false;
 }
+void OutputFilter::Kill(int sig)
+{
+   if(w)
+      w->Kill(sig);
+   if(second)
+      second->Kill(sig);
+}
+bool OutputFilter::usesfd(int n_fd)
+{
+   if(FDStream::usesfd(n_fd))
+      return true;
+   if(second_fd!=-1 && n_fd==second_fd)
+      return true;
+   if(second)
+      return second->usesfd(n_fd);
+   return n_fd<=2;
+}
 
 void FileStream::setmtime(time_t t)
 {
@@ -365,3 +400,14 @@ off_t FileStream::get_size()
    }
    return st.st_size;
 }
+
+#include "SMTask.h"
+bool FDStream::NonFatalError(int err)
+{
+   bool non_fatal=SMTask::NonFatalError(err);
+   if(non_fatal)
+      set_status(strerror(err));
+   else
+      clear_status();
+   return non_fatal;
+}
