@@ -38,7 +38,7 @@
 
 static FileInfo *ls_to_FileInfo(char *line);
 
-void Fish::GetBetterConnection(int level,int count)
+void Fish::GetBetterConnection(int level)
 {
    for(FA *fo=FirstSameSite(); fo!=0; fo=NextSameSite(fo))
    {
@@ -56,10 +56,6 @@ void Fish::GetBetterConnection(int level,int count)
 	 o->Disconnect();
 	 return;
       }
-      else
-      {
-	 takeover_time=now;
-      }
 
       if(home && !o->home)
 	 o->home=xstrdup(home);
@@ -82,7 +78,6 @@ int Fish::Do()
 {
    int m=STALL;
    int fd;
-   int count;
 
    // check if idle time exceeded
    if(mode==CLOSED && send_buf && idle>0)
@@ -132,12 +127,11 @@ int Fish::Do()
 
       // walk through Fish classes and try to find identical idle session
       // first try "easy" cases of session take-over.
-      count=CountConnections();
       for(int i=0; i<3; i++)
       {
-	 if(i>=2 && (connection_limit==0 || connection_limit>count))
+	 if(i>=2 && (connection_limit==0 || connection_limit>CountConnections()))
 	    break;
-	 GetBetterConnection(i,count);
+	 GetBetterConnection(i);
 	 if(state!=DISCONNECTED)
 	    return MOVED;
       }
@@ -152,7 +146,7 @@ int Fish::Do()
       char *init=alloca_strdup2("echo FISH:;",strlen(shell));
       strcat(init,shell);
 
-      ArgV *cmd=new ArgV("ssh");
+      ArgV *cmd=new ArgV(Query("connect-program",hostname));
       cmd->Add("-x");	// don't forward X11
       cmd->Add("-a");	// don't forward AuthAgent.
       if(user)
@@ -167,9 +161,11 @@ int Fish::Do()
       }
       cmd->Add(hostname);
       cmd->Add(init);
+      {
       char *cmd_str=cmd->Combine(0);
-      Log::global->Format(9,"---- %s (%s)\n",_("Running ssh..."),cmd_str);
+      Log::global->Format(9,"---- %s (%s)\n",_("Running connect program"),cmd_str);
       xfree(cmd_str);
+      }
       ssh=new PtyShell(cmd);
       state=CONNECTING;
       event_time=now;
@@ -755,7 +751,7 @@ int Fish::HandleReplies()
 	    if(opt_size)
 	       *opt_size=entity_size;
 	 }
-	 if(fi->defined&(fi->DATE|fi->DATE_UNPREC))
+	 if(fi->defined&fi->DATE)
 	 {
 	    entity_date=fi->date;
 	    if(opt_date)
@@ -771,7 +767,7 @@ int Fish::HandleReplies()
 	 array_for_info[array_ptr].size=fi->size;
       else
 	 array_for_info[array_ptr].size=NO_SIZE;
-      if(fi && fi->defined&(fi->DATE|fi->DATE_UNPREC))
+      if(fi && fi->defined&fi->DATE)
 	 array_for_info[array_ptr].time=fi->date;
       else
 	 array_for_info[array_ptr].time=NO_DATE;
@@ -1143,13 +1139,14 @@ DirList *Fish::MakeDirList(ArgV *args)
 {
    return new FishDirList(args,this);
 }
+#include "Glob.h"
 Glob *Fish::MakeGlob(const char *pattern)
 {
-   return new FishGlob(this,pattern);
+   return new GenericGlob(this,pattern);
 }
-ListInfo *Fish::MakeListInfo()
+ListInfo *Fish::MakeListInfo(const char *p)
 {
-   return new FishListInfo(this);
+   return new FishListInfo(this,p);
 }
 
 #undef super
@@ -1279,6 +1276,7 @@ static FileInfo *ls_to_FileInfo(char *line)
    char *sym_link=0;
    long long size;
    int n;
+   int prec=0;
 
    n=sscanf(line,"%11s %d %31s %31s %lld %3s %2d %5s%n",perms,&n_links,
 	 user,group,&size,month_name,&day,year_or_time,&consumed);
@@ -1324,7 +1322,12 @@ static FileInfo *ls_to_FileInfo(char *line)
       {
 	 sprintf(month_name,"%02d",month+1);
 	 if(year==-1)
-	    year=guess_year(month,day,hour,minute,SMTask::now);
+	    year=guess_year(month,day,hour,minute);
+	 else
+	 {
+	    hour=12;
+	    prec=12*60*60;
+	 }
       }
 
       FileInfo *fi=new FileInfo;
@@ -1347,10 +1350,13 @@ static FileInfo *ls_to_FileInfo(char *line)
 	 date.tm_isdst=-1;
 	 date.tm_sec=0;
 
-	 fi->SetDateUnprec(mktime_from_utc(&date));
+	 fi->SetDate(mktime_from_utc(&date),prec);
       }
 
       fi->SetSize(size);
+      fi->SetUser(user);
+      fi->SetGroup(group);
+      fi->SetNlink(n_links);
       fi->SetMode(perms_code);
 
       return fi;
@@ -1382,13 +1388,10 @@ static FileSet *ls_to_FileSet(const char *b,int len)
    return set;
 }
 
-// FishGlob implementation
-GenericParseGlob *FishGlob::MakeUpDirGlob()
-{
-   return new FishGlob(session,dir);
-}
-FileSet *FishGlob::Parse(const char *b,int len)
+FileSet *Fish::ParseLongList(const char *b,int len,int *err)
 {
+   if(err)
+      *err=0;
    return ls_to_FileSet(b,len);
 }
 
