@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Http.cc,v 1.137.2.2 2002/01/21 16:18:35 lav Exp $ */
+/* $Id: Http.cc,v 1.143 2002/01/21 16:25:15 lav Exp $ */
 
 #include <config.h>
 #include "trio.h"
@@ -524,7 +524,10 @@ add_path:
       else if(mode==LONG_LIST)
 	 goto retrieve;
       else if(mode==MAKE_DIR)
+      {
 	 SendMethod("PUT",efile);   // hope it would work
+	 Send("Content-length: 0\r\n");
+      }
       break;
 
    case(REMOVE):
@@ -705,7 +708,7 @@ static const char *find_eol(const char *str,int len)
    return 0;
 }
 
-void Http::GetBetterConnection(int level,int count)
+void Http::GetBetterConnection(int level)
 {
    if(level==0)
       return;
@@ -728,10 +731,6 @@ void Http::GetBetterConnection(int level,int count)
 	 o->Disconnect();
 	 return;
       }
-      else
-      {
-	 takeover_time=now;
-      }
 
       if(!peer && o->peer)
       {
@@ -754,7 +753,6 @@ int Http::Do()
    int res;
    const char *buf;
    int len;
-   int count;
 
    // check if idle time exceeded
    if(mode==CLOSED && sock!=-1 && idle>0)
@@ -847,12 +845,11 @@ int Http::Do()
 
       // walk through Http classes and try to find identical idle session
       // first try "easy" cases of session take-over.
-      count=CountConnections();
       for(int i=0; i<3; i++)
       {
-	 if(i>=2 && (connection_limit==0 || connection_limit>count))
+	 if(i>=2 && (connection_limit==0 || connection_limit>CountConnections()))
 	    break;
-	 GetBetterConnection(i,count);
+	 GetBetterConnection(i);
 	 if(state!=DISCONNECTED)
 	    return MOVED;
       }
@@ -882,12 +879,8 @@ int Http::Do()
 	    retries--;
 	    return MOVED;
 	 }
-	 if(errno==ENFILE || errno==EMFILE)
-	 {
-	    // file table overflow - it could free sometime
-	    TimeoutS(1);
+	 if(NonFatalError(errno))
 	    return m;
-	 }
 	 char str[256];
 	 sprintf(str,_("cannot create socket of address family %d"),
 			peer[peer_curr].sa.sa_family);
@@ -1314,12 +1307,8 @@ int Http::Do()
    return m;
 
 system_error:
-   if(errno==ENFILE || errno==EMFILE)
-   {
-      // file table overflow - it could free sometime
-      Timeout(1000);
+   if(NonFatalError(errno))
       return m;
-   }
    SetError(SEE_ERRNO,0);
    Disconnect();
    return MOVED;
@@ -1535,7 +1524,7 @@ int Http::Write(const void *buf,int size)
    int res=write(sock,buf,size);
    if(res==-1)
    {
-      if(E_RETRY(errno))
+      if(NonFatalError(errno))
 	 return DO_AGAIN;
       if(NotSerious(errno) || errno==EPIPE)
       {
@@ -1679,13 +1668,14 @@ DirList *Http::MakeDirList(ArgV *args)
 {
    return new HttpDirList(args,this);
 }
+#include "Glob.h"
 Glob *Http::MakeGlob(const char *pattern)
 {
-   return new HttpGlob(this,pattern);
+   return new GenericGlob(this,pattern);
 }
-ListInfo *Http::MakeListInfo()
+ListInfo *Http::MakeListInfo(const char *path)
 {
-   return new HttpListInfo(this);
+   return new HttpListInfo(this,path);
 }
 
 bool Http::CookieClosureMatch(const char *closure_c,
