@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Job.cc,v 1.22 2001/11/08 17:08:07 lav Exp $ */
+/* $Id: Job.cc,v 1.24 2002/01/31 15:30:45 lav Exp $ */
 
 #include <config.h>
 #include <stdlib.h>
@@ -516,10 +516,8 @@ void Job::ShowRunStatus(StatusLine *sl)
       j=waiting[(now/3)%waiting_num];
       current->TimeoutS(3);
    }
-   // let the job do something. Otherwise we can get 'idle' status.
-   if(j->Do()==MOVED)
-      current->Timeout(0);
-   j->ShowRunStatus(sl);
+   if(j!=this)
+      j->ShowRunStatus(sl);
 }
 
 Job *Job::FindAnyChild()
@@ -537,3 +535,13 @@ void Job::lftpMovesToBackground_ToAll()
    for(Job *scan=chain; scan; scan=scan->next)
       scan->lftpMovesToBackground();
 }
+
+bool Job::CheckForWaitLoop(Job *parent)
+{
+   if(parent==this)
+      return true;
+   for(int i=0; i<waiting_num; i++)
+      if(waiting[i]->CheckForWaitLoop(parent))
+	 return true;
+   return false;
+}
