@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-1997 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2002 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,10 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: LocalAccess.cc,v 1.32 2001/10/03 08:13:38 lav Exp $ */
-
-/* this is not very useful, just a proof of concept */
-/* this is used in mirror command to build local files list */
+/* $Id: LocalAccess.cc,v 1.40 2002/01/10 16:00:54 lav Exp $ */
 
 #include <config.h>
 #include <errno.h>
@@ -39,6 +36,7 @@
 #include "xstring.h"
 #include "misc.h"
 #include "log.h"
+#include "LocalDir.h"
 
 FileAccess *LocalAccess::New() { return new LocalAccess(); }
 
@@ -64,6 +62,8 @@ LocalAccess::LocalAccess() : FileAccess()
    Init();
    xfree(cwd);
    cwd=xgetcwd();
+   if(!cwd)
+      cwd=xstrdup(".");
 }
 LocalAccess::LocalAccess(const LocalAccess *o) : FileAccess(o)
 {
@@ -71,20 +71,21 @@ LocalAccess::LocalAccess(const LocalAccess *o) : FileAccess(o)
 }
 LocalAccess::~LocalAccess()
 {
-   if(stream)
-      delete stream;
+   delete stream;
 }
 
 void LocalAccess::errno_handle()
 {
+   int e=errno;
    xfree(error);
-   const char *err=strerror(errno);
+   const char *err=strerror(e);
    error=(char*)xmalloc(xstrlen(file)+xstrlen(file1)+strlen(err)+20);
    if(mode==RENAME)
       sprintf(error,"rename(%s, %s): %s",file,file1,err);
    else
       sprintf(error,"%s: %s",file,err);
-   Log::global->Format(0,"**** %s\n",error);
+   if(e!=EEXIST)
+      Log::global->Format(0,"**** %s\n",error);
 }
 
 int LocalAccess::Done()
@@ -158,7 +159,16 @@ int LocalAccess::Do()
       Block(stream->getfd(),POLLIN);
       return m;
    case(CHANGE_DIR):
-      if(access(file,X_OK)==-1)
+   {
+      LocalDirectory old_cwd;
+      old_cwd.SetFromCWD();
+      const char *err=old_cwd.Chdir();
+      if(err)
+      {
+	 SetError(NO_FILE,err);
+	 return MOVED;
+      }
+      if(chdir(file)==-1)
       {
 	 errno_handle();
 	 error_code=NO_FILE;
@@ -167,9 +177,11 @@ int LocalAccess::Do()
       {
 	 xfree(cwd);
 	 cwd=xstrdup(file);
+	 old_cwd.Chdir();
       }
       done=true;
       return MOVED;
+   }
    case(REMOVE): {
       const char *f=dir_file(cwd,file);
       Log::global->Format(5,"---- remove(%s)\n",f);
@@ -202,7 +214,21 @@ int LocalAccess::Do()
       return MOVED;
    }
    case(MAKE_DIR):
-      if(mkdir(dir_file(cwd,file),0755)==-1)
+      if(mkdir_p)
+      {
+	 char *sl=strchr(file,'/');
+	 while(sl)
+	 {
+	    if(sl>file)
+	    {
+	       *sl=0;
+	       mkdir(dir_file(cwd,file),0775);
+	       *sl='/';
+	    }
+	    sl=strchr(sl+1,'/');
+	 }
+      }
+      if(mkdir(dir_file(cwd,file),0775)==-1)
       {
 	 errno_handle();
 	 error_code=NO_FILE;
@@ -249,8 +275,8 @@ int LocalAccess::Do()
 	 struct stat st;
 	 if(fstat(stream->getfd(),&st)==-1)
 	 {
-	    if(opt_size) *opt_size=-1L;
-	    if(opt_date) *opt_date=(time_t)-1;
+	    if(opt_size) *opt_size=NO_SIZE;
+	    if(opt_date) *opt_date=NO_DATE;
 	 }
 	 else
 	 {
@@ -288,8 +314,8 @@ void LocalAccess::fill_array_info()
       }
       else
       {
-	 f->size=-1L;
-	 f->time=(time_t)-1;
+	 f->size=NO_SIZE;
+	 f->time=NO_DATE;
       }
    }
 }
@@ -323,11 +349,12 @@ read_again:
 
    if(res<0)
    {
-      if(E_RETRY(errno))
+      if(stream->NonFatalError(errno))
 	 return DO_AGAIN;
       saved_errno=errno;
       return SEE_ERRNO;
    }
+   stream->clear_status();
    if(res==0)
       return res; // eof
 
@@ -424,11 +451,12 @@ int LocalAccess::Write(const void *vbuf,int len)
    int res=write(fd,buf,len);
    if(res<0)
    {
-      if(E_RETRY(errno))
+      if(stream->NonFatalError(errno))
 	 return DO_AGAIN;
       saved_errno=errno;
       return SEE_ERRNO;
    }
+   stream->clear_status();
 
    if(res==len)
       res+=skip_cr;
@@ -438,16 +466,20 @@ int LocalAccess::Write(const void *vbuf,int len)
 
 int LocalAccess::StoreStatus()
 {
-   if(stream)
+   if(!stream)
+      return IN_PROGRESS;
+   if(stream->getfd()==-1)
    {
-      delete stream;
-      stream=0;
-      if(entity_date!=(time_t)-1)
-      {
-	 static struct utimbuf ut;
-	 ut.actime=ut.modtime=entity_date;
-	 utime(dir_file(cwd,file),&ut);
-      }
+      if(stream->error())
+	 SetError(NO_FILE,stream->error_text);
+   }
+   delete stream;
+   stream=0;
+   if(error_code==OK && entity_date!=NO_DATE)
+   {
+      static struct utimbuf ut;
+      ut.actime=ut.modtime=entity_date;
+      utime(dir_file(cwd,file),&ut);
    }
 
    if(error_code<0)
@@ -460,14 +492,18 @@ void LocalAccess::Close()
 {
    done=false;
    error_code=OK;
-   if(stream)
-   {
-      delete stream;
-      stream=0;
-   }
+   delete stream;
+   stream=0;
    FileAccess::Close();
 }
 
+const char *LocalAccess::CurrentStatus()
+{
+   if(stream && stream->status)
+      return stream->status;
+   return "";
+}
+
 bool LocalAccess::SameLocationAs(FileAccess *fa)
 {
    if(!SameProtoAs(fa))
@@ -480,9 +516,9 @@ bool LocalAccess::SameLocationAs(FileAccess *fa)
    return !xstrcmp(cwd,o->cwd);
 }
 
-ListInfo *LocalAccess::MakeListInfo()
+ListInfo *LocalAccess::MakeListInfo(const char *path)
 {
-   return new LocalListInfo(cwd);
+   return new LocalListInfo(this,path);
 }
 
 int LocalListInfo::Do()
@@ -490,6 +526,7 @@ int LocalListInfo::Do()
    if(done)
       return STALL;
 
+   const char *dir=session->GetCwd();
    DIR *d=opendir(dir);
    struct dirent *f;
 
@@ -508,25 +545,34 @@ int LocalListInfo::Do()
       f=readdir(d);
       if(f==0)
 	 break;
-      FileInfo *fi=new FileInfo();
-      fi->SetName(f->d_name);
-      result->Add(fi);
+      result->Add(new FileInfo(f->d_name));
    }
    closedir(d);
 
-   result->Exclude(path,rxc_exclude,rxc_include);
+   result->Exclude(exclude_prefix,exclude);
 
    result->rewind();
    for(FileInfo *file=result->curr(); file!=0; file=result->next())
    {
       const char *name=dir_file(dir,file->name);
       file->LocalFile(name, follow_symlinks);
+      if(!(file->defined&file->TYPE))
+	 result->SubtractCurr();
    }
 
    done=true;
    return MOVED;
 }
 
+#include "Glob.h"
+class LocalGlob : public Glob
+{
+   const char *cwd;
+public:
+   LocalGlob(const char *cwd,const char *pattern);
+   const char *Status() { return "..."; }
+   int Do();
+};
 Glob *LocalAccess::MakeGlob(const char *pattern)
 {
    xfree(file);
@@ -546,8 +592,10 @@ int LocalGlob::Do()
       return STALL;
 
    glob_t g;
-   char *oldcwd=xgetcwd();
-   if(!oldcwd)
+   LocalDirectory oldcwd;
+   oldcwd.SetFromCWD();
+   // check if we can return.
+   if(oldcwd.Chdir())
    {
       SetError(_("cannot get current directory"));
       return MOVED;
@@ -558,7 +606,6 @@ int LocalGlob::Do()
       char *err=(char*)alloca(strlen(cwd)+strlen(se)+20);
       sprintf(err,"chdir(%s): %s",cwd,se);
       SetError(err);
-      xfree(oldcwd);
       return MOVED;
    }
 
@@ -583,9 +630,10 @@ int LocalGlob::Do()
    }
    globfree(&g);
 
-   if(chdir(oldcwd)==-1)
-      fprintf(stderr,"chdir(%s): %s",oldcwd,strerror(errno));
-   xfree(oldcwd);
+   const char *err=oldcwd.Chdir();
+   const char *name=oldcwd.GetName();
+   if(err)
+      fprintf(stderr,"chdir(%s): %s",name?name:"?",err);
 
    done=true;
    return MOVED;
