@@ -18,9 +18,10 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: LsCache.cc,v 1.12 2000/03/22 14:12:48 lav Exp $ */
+/* $Id: LsCache.cc,v 1.18 2001/12/24 06:24:04 lav Exp $ */
 
 #include <config.h>
+#include <assert.h>
 #include "LsCache.h"
 #include "xmalloc.h"
 #include "plural.h"
@@ -32,6 +33,11 @@ long	 LsCache::sizelimit=1024*1024;
 TimeInterval LsCache::ttl("60m");  // time to live = 60 minutes
 LsCache::ExpireHelper LsCache::expire_helper;
 
+FileSet *LsCache::fset=0;
+FileAccess *LsCache::fset_loc=0;
+int LsCache::fset_m=0;
+char *LsCache::fset_a=0;
+
 void LsCache::CheckSize()
 {
    if(sizelimit<0)
@@ -91,6 +97,8 @@ void LsCache::Add(FileAccess *p_loc,const char *a,int m,const char *d,int l)
    else
    {
       xfree(scan->data);
+      if(fset_loc && scan->loc->SameLocationAs(fset_loc))
+	 free_fset();
    }
    scan->data=(char*)xmemdup(d,l);
    scan->data_len=l;
@@ -117,7 +125,7 @@ int LsCache::Find(FileAccess *p_loc,const char *a,int m,const char **d,int *l)
    LsCache *scan;
    for(scan=chain; scan; scan=scan->next)
    {
-      if(scan->mode==m && !strcmp(scan->arg,a) && p_loc->SameLocationAs(scan->loc))
+      if((m == -1 || scan->mode==m) && !strcmp(scan->arg,a) && p_loc->SameLocationAs(scan->loc))
       {
 	 if(d && l)
 	 {
@@ -130,10 +138,47 @@ int LsCache::Find(FileAccess *p_loc,const char *a,int m,const char **d,int *l)
    return 0;
 }
 
+FileSet *LsCache::FindFileSet(FileAccess *p_loc,const char *a,int m)
+{
+   if(fset && m==fset_m && !strcmp(fset_a,a) && p_loc->SameLocationAs(fset_loc))
+      return fset;
+
+   const char *buf_c;
+   int bufsiz;
+   if(!Find(p_loc, a, m, &buf_c, &bufsiz))
+      return 0;
+
+   free_fset();
+
+   FileSet *new_fset=p_loc->ParseLongList(buf_c, bufsiz);
+   if(!new_fset)
+      return 0;
+
+   fset=new_fset;
+   fset_a=xstrdup(a);
+   fset_m=m;
+   fset_loc=p_loc->Clone();
+
+   return fset;
+}
+
+void LsCache::free_fset()
+{
+   SMTask::Delete(fset_loc);
+   xfree(fset_a);
+   delete fset;
+
+   fset_loc=0;
+   fset_a=0;
+   fset=0;
+}
+
 LsCache::~LsCache()
 {
    if(expire_helper.expiring==this)
       expire_helper.expiring=0;
+   if(fset_loc && loc->SameLocationAs(fset_loc))
+      free_fset();
    SMTask::Delete(loc);
    xfree(data);
    xfree(arg);
@@ -243,3 +288,82 @@ void LsCache::Changed(change_mode m,FileAccess *f,const char *dir)
       scan=&scan[0]->next;
    }
 }
+
+/* Mark a path as a directory or file. (We have other ways of knowing this;
+ * this is the most explicit and least expensive.) */
+void LsCache::SetDirectory(FileAccess *p_loc, const char *path, bool dir)
+{
+   if(!path)
+      return;
+
+   char *origdir = alloca_strdup(p_loc->GetCwd());
+
+   p_loc->Chdir(path,false);
+   const char *entry = dir? "1":"0";
+   LsCache::Add(p_loc,"",FileAccess::CHANGE_DIR, entry, strlen(entry));
+   p_loc->Chdir(origdir,false);
+}
+
+/* This is a hint function. If file type is really needed, use GetFileInfo
+ * with showdir set to true. (GetFileInfo uses this function.)
+ * Returns -1 if type is not known, 1 if a directory, 0 if a file. */
+
+int LsCache::IsDirectory(FileAccess *p_loc,const char *dir_c)
+{
+   if(*dir_c && dir_c[strlen(dir_c)-1] == '/')
+      return 1;
+
+   char *origdir = alloca_strdup(p_loc->GetCwd());
+   p_loc->Chdir(dir_c, false);
+
+   int ret = -1;
+
+   /* Cheap tests first:
+    *
+    * First, we know the path is a directory or not if we have an expicit
+    * CHANGE_DIR entry for it. */
+   const char *buf_c;
+   int bufsiz;
+   if(Find(p_loc, "", FileAccess::CHANGE_DIR, &buf_c,&bufsiz))
+   {
+      ret = (buf_c[0]=='1');
+      goto leave;
+   }
+
+   /* We know the path is a directory if we have a cache entry for it.  This is
+    * true regardless of the list type.  (Unless it's a CHANGE_DIR entry; do this
+    * test after the CHANGE_DIR check.) */
+   if(Find(p_loc, "", -1, 0,0))
+   {
+      ret = 1;
+      goto leave;
+   }
+
+   /* We know this is a file or a directory if the dirname is cached and
+    * contains the basename. */
+   {
+      p_loc->SetCwd(origdir);
+      char *dir = alloca_strdup(dir_c);
+      char *sl = strrchr(dir, '/');
+      if(sl)
+      {
+	 if(sl>dir)
+	    *sl=0;
+	 else
+	    sl[1]=0;
+	 p_loc->Chdir(dir, false);
+      }
+
+      const FileSet *fs=FindFileSet(p_loc, "", FA::LONG_LIST);
+      if(fs)
+      {
+	 FileInfo *fi=fs->FindByName(basename_ptr(dir_c));
+	 if(fi && (fi->defined&fi->TYPE))
+	    ret = (fi->filetype == fi->DIRECTORY);
+      }
+   }
+
+leave:
+   p_loc->SetCwd(origdir);
+   return ret;
+}
