@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: NetAccess.cc,v 1.44.2.1 2001/11/21 09:59:20 lav Exp $ */
+/* $Id: NetAccess.cc,v 1.52 2001/12/20 11:14:44 lav Exp $ */
 
 #include <config.h>
 
@@ -30,6 +30,7 @@
 #include "url.h"
 #include "LsCache.h"
 #include "misc.h"
+#include "Speedometer.h"
 
 #define super FileAccess
 
@@ -61,7 +62,6 @@ void NetAccess::Init()
 
    connection_limit=0;	// no limit.
    connection_takeover=false;
-   takeover_time=NO_DATE;
 
    Reconfig(0);
 }
@@ -490,7 +490,7 @@ const char *NetAccess::DelayingMessage()
 {
    static char buf[80];
    if(connection_limit>0 && connection_limit<=CountConnections())
-      return _("Connections limit reached");
+      return _("Connection limit reached");
    long remains=ReconnectInterval()-(time_t(now)-try_time);
    if(remains<=0)
       return "";
@@ -523,7 +523,6 @@ void NetAccess::Close()
 
    retries=0;
    persist_retries=0;
-   takeover_time=NO_DATE;
 
    Delete(resolver);
    resolver=0;
@@ -543,159 +542,6 @@ int NetAccess::CountConnections()
 }
 
 
-// GenericParseGlob implementation
-GenericParseGlob::GenericParseGlob(FileAccess *s,const char *n_pattern)
-   : Glob(n_pattern)
-{
-   curr_dir=0;
-   dir_list=0;
-   dir_index=0;
-   updir_glob=0;
-   ubuf=0;
-
-   session=s;
-   dir=xstrdup(pattern);
-   char *slash=strrchr(dir,'/');
-   if(!slash)
-      dir[0]=0;	  // current directory
-   else if(slash>dir)
-      *slash=0;	  // non-root directory
-   else
-      dir[1]=0;	  // root directory
-
-   if(done)
-      return;
-}
-
-GenericParseGlob::~GenericParseGlob()
-{
-   Delete(updir_glob);
-   Delete(ubuf);
-   xfree(dir);
-}
-
-int GenericParseGlob::Do()
-{
-   int   m=STALL;
-
-   if(done)
-      return m;
-
-   if(dir[0] && !updir_glob && !dir_list)
-   {
-      updir_glob=MakeUpDirGlob();
-      updir_glob->DirectoriesOnly();
-   }
-
-   if(updir_glob && !dir_list)
-   {
-      if(updir_glob->Error())
-      {
-	 SetError(updir_glob->ErrorText());
-	 return MOVED;
-      }
-      if(!updir_glob->Done())
-	 return m;
-      dir_list=updir_glob->GetResult();
-      dir_index=0;
-      m=MOVED;
-   }
-
-   if(!ubuf)
-   {
-      curr_dir=0;
-      if(dir_list && (*dir_list)[dir_index])
-	 curr_dir=(*dir_list)[dir_index]->name;
-      else if(dir_index==0)
-	 curr_dir=dir;
-      if(curr_dir==0) // all done
-      {
-	 done=true;
-	 return MOVED;
-      }
-
-      const char *cache_buffer=0;
-      int cache_buffer_size=0;
-      if(use_cache && LsCache::Find(session,curr_dir,FA::LONG_LIST,
-				    &cache_buffer,&cache_buffer_size))
-      {
-	 ubuf=new Buffer();
-	 ubuf->Put(cache_buffer,cache_buffer_size);
-	 ubuf->PutEOF();
-      }
-      else
-      {
-	 session->Open(curr_dir,FA::LONG_LIST);
-	 session->UseCache(use_cache);
-	 session->RereadManual();
-	 ubuf=new IOBufferFileAccess(session);
-	 if(LsCache::IsEnabled())
-	    ubuf->Save(LsCache::SizeLimit());
-      }
-      m=MOVED;
-   }
-
-   if(ubuf->Error())
-   {
-      SetError(ubuf->ErrorText());
-      Delete(ubuf);
-      ubuf=0;
-      return MOVED;
-   }
-
-   if(!ubuf->Eof())
-   {
-      if(session->GetRealPos()==0 && session->GetPos()>0)
-      {
-	 session->SeekReal();
-	 ubuf->Empty();
-	 return MOVED;
-      }
-      return m;
-   }
-
-   LsCache::Add(session,curr_dir,FA::LONG_LIST,ubuf);
-
-   // now we have all the index in ubuf; parse it.
-   const char *b;
-   int len;
-   ubuf->Get(&b,&len);
-
-   FileSet *set=Parse(b,len);
-
-   Delete(ubuf);
-   ubuf=0;
-
-   set->rewind();
-   for(FileInfo *f=set->curr(); f; f=set->next())
-   {
-      f->SetName(dir_file(curr_dir,f->name));
-      add(f);
-   }
-
-   delete set;
-
-   dir_index++;
-   m=MOVED;
-
-   return m;
-}
-
-const char *GenericParseGlob::Status()
-{
-   if(updir_glob && !dir_list)
-      return updir_glob->Status();
-
-   static char s[256];
-   if(ubuf && !ubuf->Eof() && session->IsOpen())
-   {
-      sprintf(s,_("Getting file list (%lld) [%s]"),
-		     (long long)session->GetPos(),session->CurrentStatus());
-      return s;
-   }
-   return "";
-}
-
 // GenericParseListInfo implementation
 int GenericParseListInfo::Do()
 {
@@ -714,7 +560,7 @@ int GenericParseListInfo::Do()
    {
       const char *cache_buffer=0;
       int cache_buffer_size=0;
-      if(use_cache && LsCache::Find(session,"",FA::LONG_LIST,
+      if(use_cache && LsCache::Find(session,"",mode,
 				    &cache_buffer,&cache_buffer_size))
       {
 	 ubuf=new Buffer();
@@ -723,9 +569,10 @@ int GenericParseListInfo::Do()
       }
       else
       {
-	 session->Open("",FA::LONG_LIST);
+	 session->Open("",mode);
 	 session->UseCache(use_cache);
 	 ubuf=new IOBufferFileAccess(session);
+	 ubuf->SetSpeedometer(new Speedometer());
 	 if(LsCache::IsEnabled())
 	    ubuf->Save(LsCache::SizeLimit());
       }
@@ -744,22 +591,37 @@ int GenericParseListInfo::Do()
       if(!ubuf->Eof())
 	 return m;
 
-      LsCache::Add(session,"",FA::LONG_LIST,ubuf);
+      LsCache::Add(session,"",mode,ubuf);
 
       // now we have all the index in ubuf; parse it.
       const char *b;
       int len;
       ubuf->Get(&b,&len);
 
-      result=Parse(b,len);
+      int old_mode=mode;
+
+      FileSet *set=Parse(b,len);
+      if(result)
+      {
+	 result->Merge(set);
+	 delete set;
+      }
+      else
+	 result=set;
 
       Delete(ubuf);
       ubuf=0;
       m=MOVED;
 
+      // try another mode? Parse() can set mode to indicate it wants to try it.
+      if(mode!=old_mode)
+	 return m;
+
+      if(!result)
+	 result=new FileSet;
+
       result->ExcludeDots();
-      if(rxc_exclude || rxc_include)
-	 result->Exclude(path,rxc_exclude,rxc_include);
+      result->Exclude(exclude_prefix,exclude);
 
       get_info_cnt=result->get_fnum();
       if(get_info_cnt==0)
@@ -775,9 +637,9 @@ int GenericParseListInfo::Do()
       result->rewind();
       for(file=result->curr(); file!=0; file=result->next())
       {
-	 cur->get_size = !(file->defined & file->SIZE) && need_size;
-	 cur->get_time = !(file->defined & file->DATE) && need_time
-	       && (!(file->defined & file->DATE_UNPREC) || can_get_prec_time);
+	 cur->get_size = need_size && !(file->defined & file->SIZE);
+	 cur->get_time = need_time && (!(file->defined & file->DATE)
+				 || (file->date_prec>0 && can_get_prec_time));
 	 cur->file=0;
 
 	 if(file->defined & file->TYPE)
@@ -785,7 +647,7 @@ int GenericParseListInfo::Do()
 	    if(file->filetype==file->SYMLINK && follow_symlinks)
 	    {
 	       file->filetype=file->NORMAL;
-	       file->defined &= ~(file->SIZE|file->SYMLINK_DEF|file->MODE|file->DATE_UNPREC);
+	       file->defined &= ~(file->SIZE|file->SYMLINK_DEF|file->MODE|file->DATE);
 	       cur->get_size=true;
 	       cur->get_time=true;
 	    }
@@ -839,7 +701,7 @@ int GenericParseListInfo::Do()
       for(cur=get_info; get_info_cnt-->0; cur++)
       {
 	 if(cur->time!=(time_t)-1)
-	    result->SetDate(cur->file,cur->time);
+	    result->SetDate(cur->file,cur->time,0);
 	 if(cur->size!=-1)
 	    result->SetSize(cur->file,cur->size);
       }
@@ -851,10 +713,9 @@ int GenericParseListInfo::Do()
    return m;
 }
 
-GenericParseListInfo::GenericParseListInfo(FileAccess *s)
+GenericParseListInfo::GenericParseListInfo(FileAccess *s,const char *p)
+   : ListInfo(s,p)
 {
-   session=s;
-
    get_info=0;
    get_info_cnt=0;
 
@@ -862,11 +723,12 @@ GenericParseListInfo::GenericParseListInfo(FileAccess *s)
 
    get_time_for_dirs=true;
    can_get_prec_time=true;
+
+   mode=FA::LONG_LIST;
 }
 
 GenericParseListInfo::~GenericParseListInfo()
 {
-   session->Close();
    xfree(get_info);
    Delete(ubuf);
 }
@@ -876,14 +738,16 @@ const char *GenericParseListInfo::Status()
    static char s[256];
    if(ubuf && !ubuf->Eof() && session->IsOpen())
    {
-      sprintf(s,_("Getting directory contents (%lld)"),
-		     (long long)session->GetPos());
+      sprintf(s,_("Getting directory contents (%lld) %s[%s]"),
+		     (long long)session->GetPos(),
+		     ubuf->GetRateStrS(),session->CurrentStatus());
       return s;
    }
    if(get_info)
    {
-      sprintf(s,_("Getting files information (%d%%)"),
-		     session->InfoArrayPercentDone());
+      sprintf(s,_("Getting files information (%d%%) [%s]"),
+		     session->InfoArrayPercentDone(),
+		     session->CurrentStatus());
       return s;
    }
    return "";
