@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: SMTask.cc,v 1.20.2.1 2002/01/15 16:20:23 lav Exp $ */
+/* $Id: SMTask.cc,v 1.23 2002/01/31 15:30:45 lav Exp $ */
 
 #include <config.h>
 #include <assert.h>
@@ -73,14 +73,14 @@ SMTask::~SMTask()
 #endif
    task_count--;
    assert(!running);
+   if(this==sched_scan)
+      sched_scan=0;
    // remove from the chain
    SMTask **scan=&chain;
    while(*scan)
    {
       if(*scan==this)
       {
-	 assert(this!=sched_scan);
-
 	 *scan=next;
 	 break;
       }
@@ -234,3 +234,35 @@ int SMTask::TaskCount()
       count++;
    return count;
 }
+
+#include <errno.h>
+#include "ResMgr.h"
+ResDecl enospc_fatal ("xfer:disk-full-fatal","no",ResMgr::BoolValidate,ResMgr::NoClosure);
+bool SMTask::NonFatalError(int err)
+{
+   if(E_RETRY(err))
+      return true;
+
+   current->TimeoutS(1);
+   if(err==ENFILE || err==EMFILE)
+      return true;
+#ifdef ENOBUFS
+   if(err==ENOBUFS)
+      return true;
+#endif
+#ifdef ENOSR
+   if(err==ENOSR)
+      return true;
+#endif
+#ifdef ENOSPC
+   if(err==ENOSPC)
+      return !enospc_fatal.QueryBool(0);
+#endif
+#ifdef EDQUOT
+   if(err==EDQUOT)
+      return !enospc_fatal.QueryBool(0);
+#endif
+
+   current->Timeout(0);
+   return false; /* fatal error */
+}
