@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Speedometer.cc,v 1.11 2001/07/30 14:39:14 lav Exp $ */
+/* $Id: Speedometer.cc,v 1.14 2001/11/06 15:34:56 lav Exp $ */
 
 #include <config.h>
 #include <math.h>
@@ -46,7 +46,7 @@ Speedometer::Speedometer(const char *p)
 }
 bool Speedometer::Valid()
 {
-   return now>start+1 && now<last_bytes+period;
+   return now>=start+1 && now<last_bytes+period;
 }
 int Speedometer::Do()
 {
@@ -61,15 +61,23 @@ float Speedometer::Get()
 }
 void Speedometer::Add(int b)
 {
-   float div=period;
+   double div=period;
+
    if(start>now)
       start=now;  // time was adjusted?
    if(now<last_second)
       last_second=now;
-   if(now-start<div)
-      div=now-start+1;
-   rate*=pow(double(div-1)/div,int(now-last_second));
+
+   double time_passed_since_start=TimeDiff(now,start);
+   double time_passed=TimeDiff(now,last_second);
+   if(time_passed_since_start<div)
+      div=time_passed_since_start;
+   if(div<1)
+      div=1;
+
+   rate*=1-time_passed/div;
    rate+=b/div;
+
    last_second=now;
    if(b>0)
       last_bytes=now;
@@ -120,6 +128,7 @@ const char *Speedometer::GetETAStrFromTime(long eta)
    const char minute_c=_("minute")[0];
    const char second_c=_("second")[0];
 
+   // for translator: Estimated Time of Arrival.
    const char *tr_eta=_("eta:");
 
    if(terse)
@@ -222,6 +231,6 @@ void Speedometer::Reset()
 ResDecl res_eta_terse("xfer:eta-terse",  "yes",ResMgr::BoolValidate,ResMgr::NoClosure);
 void Speedometer::Reconfig(const char *n)
 {
-   terse=res_eta_terse.Query(0);
+   terse=res_eta_terse.QueryBool(0);
    SetPeriod(ResMgr::Query(period_resource,0));
 }
