@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: buffer.h,v 1.19 2001/11/08 09:25:49 lav Exp $ */
+/* $Id: buffer.h,v 1.21 2001/12/18 10:50:39 lav Exp $ */
 
 #ifndef BUFFER_H
 #define BUFFER_H
@@ -28,8 +28,12 @@
 #include "fg.h"
 #include "xstring.h"
 
+#include <stdarg.h>
+
 #define GET_BUFSIZE 0x4000
 
+class Speedometer;
+
 class Buffer : public SMTask
 {
 protected:
@@ -52,6 +56,9 @@ protected:
    virtual int Put_LL(const char *buf,int size) { return 0; }
    virtual int PutEOF_LL() { return 0; }
 
+   Speedometer *rate;
+   void RateAdd(int n);
+
    void Allocate(int size);
 
    void SaveMaxCheck(int addsize);
@@ -76,6 +83,7 @@ public:
    void Put(const char *buf,int size);
    void Put(const char *buf) { Put(buf,strlen(buf)); }
    void Format(const char *f,...) PRINTF_LIKE(2,3);
+   void vFormat(const char *f, va_list v);
    void PutEOF() { eof=true; PutEOF_LL(); }
 
    // useful for cache.
@@ -84,6 +92,10 @@ public:
    void SaveRollback(int p);
 
    void SetPos(off_t p) { pos=p; }
+   off_t GetPos() const { return pos; }
+
+   void SetSpeedometer(Speedometer *s) { rate=s; }
+   const char *GetRateStrS();
 
    void Empty();
 
