@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2001 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2002 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: commands.cc,v 1.151.2.6 2002/01/27 08:55:48 lav Exp $ */
+/* $Id: commands.cc,v 1.176 2002/02/11 16:56:05 lav Exp $ */
 
 #include <config.h>
 
@@ -67,6 +67,8 @@
 #include "QueueFeeder.h"
 #include "lftp_rl.h"
 #include "FileSetOutput.h"
+#include "PatternSet.h"
+#include "LocalDir.h"
 
 #include "confpaths.h"
 
@@ -144,7 +146,15 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 	 "is stored as `-'. You can do `cd -' to change the directory back.\n"
 	 "The previous directory for each site is also stored on disk, so you can\n"
 	 "do `open site; cd -' even after lftp restart.\n")},
-   {"chmod",   cmd_chmod,   N_("chmod mode file..."), 0},
+   {"chmod",   cmd_chmod,   N_("chmod [OPTS] mode file..."),
+	 N_("Change the mode of each FILE to MODE.\n"
+	    "\n"
+	    " -c, --changes        - like verbose but report only when a change is made\n"
+	    " -f, --quiet          - suppress most error messages\n"
+	    " -v, --verbose        - output a diagnostic for every file processed\n"
+	    " -R, --recursive      - change files and directories recursively\n"
+	    "\n"
+	    "MODE can be an octal number or symbolic mode (see chmod(1))\n")},
    {"close",   cmd_close,   "close [-a]",
 	 N_("Close idle connections. By default only with current server.\n"
 	 " -a  close idle connections with all servers\n")},
@@ -159,7 +169,12 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 	     * them align with GNU ls options. */
 	    " -1                   - single-column output\n"
 	    " -B, --basename       - show basename of files only\n"
+	    "     --block-size=SIZ - use SIZ-byte blocks\n"
+	    " -d, --directory      - list directory entries instead of contents\n"
 	    " -F, --classify       - append indicator (one of /@) to entries\n"
+	    " -h, --human-readable - print sizes in human readable format (e.g., 1K)\n"
+	    "     --si             - likewise, but use powers of 1000 not 1024\n"
+	    " -k, --kilobytes      - like --block-size=1024\n"
 	    " -l, --long           - use a long listing format\n"
 	    " -q, --quiet          - don't show status\n"
 	    " -s, --size           - print size of each file\n"
@@ -169,6 +184,8 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 	    " -D, --dirsfirst      - list directories first\n"
 	    "     --sort=OPT       - \"name\", \"size\"\n"
 	    " -S                   - sort by file size\n"
+	    " --user, --group, --perms, --date, --linkcount, --links\n"
+	    "                      - show individual fields\n"
 	    "\n"
 	    "By default, cls output is cached, to see new listing use `recls' or\n"
 	    "`cache flush'.\n"
@@ -183,14 +200,10 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 	     * add --filesize; it won't have any effect unless -s is also used, so
 	     * it can be enabled all the time. (that's also poorly worded, and too
 	     * long.) */
-	    "Tips: Use --filesize and -D to pack the listing better.  If you don't\n"
+	    "Tips: Use --filesize with -D to pack the listing better.  If you don't\n"
 	    "always want to see file sizes, --filesize in cls-default will affect the\n"
-	    "-s flag on the commandline as well.  All flags work in\n"
-	    "cls-completion-default.  -i in cls-completion-default makes filename\n"
-	    "completion case-insensitive."
-	    "\n"
-	    "Usage note: directory names must have a trailing slash; \"cls pub/\" to\n"
-	    "be recognized as a directory.\n"
+	    "-s flag on the commandline as well.  Add `-i' to cls-completion-default\n"
+	    "to make filename completion case-insensitive.\n"
 	   )},
    {"connect", cmd_open,   0,"open"},
    {"command", cmd_command},
@@ -207,6 +220,7 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 	 "                       only if it is N or fewer levels below the command\n"
 	 "                       line argument;  --max-depth=0 is the same as\n"
 	 "                       --summarize\n"
+	 " -F, --files           print number of files instead of sizes\n"
 	 " -h, --human-readable  print sizes in human readable format (e.g., 1K 234M 2G)\n"
 	 " -H, --si              likewise, but use powers of 1000 not 1024\n"
 	 " -k, --kilobytes       like --block-size=1024\n"
@@ -292,6 +306,7 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 	 " -c, --continue         continue a mirror job if possible\n"
 	 " -e, --delete           delete files not present at remote site\n"
 	 " -s, --allow-suid       set suid/sgid bits according to remote site\n"
+	 "     --allow-chown      try to set owner and group on files\n"
 	 " -n, --only-newer       download only newer files (-c won't work)\n"
 	 " -r, --no-recursion     don't go to subdirectories\n"
 	 " -p, --no-perms         don't set file permissions\n"
@@ -303,11 +318,6 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 	 " -i RX, --include RX    include matching files\n"
 	 " -x RX, --exclude RX    exclude matching files\n"
 	 "                        RX is extended regular expression\n"
-	 " -t Nx, --time-prec Nx  set time precision to N seconds (x=s)\n"
-	 "                        minutes (x=m) hours (x=h) or days (x=d)\n"
-	 "                        default - mirror:time-precision setting\n"
-	 " -T Nx, --loose-time-prec  set time precision for imprecise time\n"
-	 "                        default - mirror:loose-time-precision\n"
 	 " -v, --verbose[=N]      verbose operation\n"
 	 "     --use-cache        use cached directory listings\n"
 	 "\n"
@@ -378,6 +388,12 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 	 "       queue --move|-m <index or wildcard expression> [index]\n\n"
 	 "Move the given items before the given queue index, or to the end if no\n"
 	 "destination is given.\n"
+	 "\n"
+	 "Options:\n"
+	 " -q                  Be quiet.\n"
+	 " -v                  Be verbose.\n"
+	 " -Q                  Output in a format that can be used to re-queue.\n"
+	 "                     Useful with --delete.\n"
 	 )},
    {"quit",    cmd_exit,   0,"exit"},
    {"quote",   cmd_ls,	   N_("quote <cmd>"),
@@ -451,6 +467,7 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
    {NULL,NULL}
 };
 
+#define charcasecmp(a,b) (tolower((unsigned char)(a))-tolower((unsigned char)(b)))
 // returns:
 //    0 - no match
 //    1 - found, if *res==0 then ambiguous
@@ -462,7 +479,7 @@ int find_command(const char *unprec_name,const char * const *names,
    for( ; *names; names++)
    {
       const char *s,*u;
-      for(s=*names,u=unprec_name; *s && *u==*s; s++,u++)
+      for(s=*names,u=unprec_name; *s && !charcasecmp(*u,*s); s++,u++)
 	 ;
       if(*s && !*u)
       {
@@ -508,7 +525,14 @@ Job *CmdExec::builtin_lcd()
 
    cd_to=expand_home_relative(cd_to);
 
-   RestoreCWD();
+   if(RestoreCWD()==-1)
+   {
+      if(cd_to[0]!='/')
+      {
+	 eprintf("No current local directory, use absolute path.\n");
+	 return 0;
+      }
+   }
 
    int res=chdir(cd_to);
    if(res==-1)
@@ -519,13 +543,13 @@ Job *CmdExec::builtin_lcd()
    }
 
    xfree(old_lcwd);
-   old_lcwd=cwd;
-   cwd=0;
+   old_lcwd=xstrdup(cwd->GetName());
 
-   SaveCWD();  // this allocates cwd again
+   SaveCWD();
 
+   const char *name=cwd->GetName();
    if(interactive)
-      eprintf(_("lcd ok, local cwd=%s\n"),cwd?cwd:"?");
+      eprintf(_("lcd ok, local cwd=%s\n"),name?name:"?");
 
    exit_code=0;
    return 0;
@@ -953,7 +977,10 @@ Job *CmdExec::builtin_queue()
    static struct option queue_options[]=
    {
       {"move",required_argument,0,'m'},
-      {"delete",optional_argument,0,'d'},
+      {"delete",no_argument,0,'d'},
+      {"quiet",no_argument,0,'q'},
+      {"verbose",no_argument,0,'v'},
+      {"queue",required_argument,0,'Q'},
       {0,0,0,0}
    };
    enum { ins, del, move } mode = ins;
@@ -961,13 +988,14 @@ Job *CmdExec::builtin_queue()
    const char *arg = NULL;
    /* position to insert at (ins only) */
    int pos = -1; /* default to the end */
+   int verbose = -1; /* default */
 
    args->rewind();
    int opt;
 
    exit_code=1; // more failure exits than success exits, so set success explicitely
 
-   while((opt=args->getopt_long("+dm:n:w",queue_options,0))!=EOF)
+   while((opt=args->getopt_long("+dm:n:qvQw",queue_options,0))!=EOF)
    {
       switch(opt)
       {
@@ -990,6 +1018,18 @@ Job *CmdExec::builtin_queue()
 	 mode = del;
 	 break;
 
+      case 'q':
+	 verbose = 0;
+	 break;
+
+      case 'v':
+	 verbose = 2;
+	 break;
+
+      case 'Q':
+	 verbose = QueueFeeder::PrintRequeue;
+	 break;
+
       case '?':
 	 err:
 	 eprintf(_("Try `help %s' for more information.\n"),args->a0());
@@ -997,6 +1037,14 @@ Job *CmdExec::builtin_queue()
       }
    }
 
+   if(verbose == -1)
+   {
+      if(mode == ins || mode == move)
+	 verbose = 0;
+      else
+	 verbose = 1;
+   }
+
    const int args_remaining = args->count() - args->getindex();
    switch(mode) {
       case ins: {
@@ -1011,7 +1059,7 @@ Job *CmdExec::builtin_queue()
 	 else
 		 cmd=args->CombineQuoted(args->getindex());
 
-	 queue->has_queue->QueueCmd(cmd, session->GetCwd(), cwd, pos);
+	 queue->has_queue->QueueCmd(cmd, session->GetCwd(), cwd?cwd->GetName():0, pos, verbose);
 	 xfree(cmd);
 
 	 last_bg=queue->jobno;
@@ -1036,11 +1084,11 @@ Job *CmdExec::builtin_queue()
 	 }
 
 	 if(!arg)
-	    queue->has_queue->DelJob(-1); /* delete the last job */
+	    queue->has_queue->DelJob(-1, verbose); /* delete the last job */
 	 else if(isdigit(arg[0]))
-	    queue->has_queue->DelJob(atoi(arg)-1);
+	    queue->has_queue->DelJob(atoi(arg)-1, verbose);
 	 else
-	    queue->has_queue->DelJob(arg);
+	    queue->has_queue->DelJob(arg, verbose);
 
 	 exit_code=0;
       }
@@ -1066,11 +1114,11 @@ Job *CmdExec::builtin_queue()
 	 }
 
 	 if(isdigit(arg[0])) {
-	    queue->has_queue->MoveJob(atoi(arg)-1, to);
+	    queue->has_queue->MoveJob(atoi(arg)-1, to, verbose);
 	    break;
 	 }
 
-	 queue->has_queue->MoveJob(arg, to);
+	 queue->has_queue->MoveJob(arg, to, verbose);
 	 exit_code=0;
       }
       break;
@@ -1079,6 +1127,7 @@ Job *CmdExec::builtin_queue()
    return 0;
 }
 
+#if 0
 Job *CmdExec::builtin_queue_edit()
 {
    CmdExec *queue=GetQueue();
@@ -1130,13 +1179,13 @@ Job *CmdExec::builtin_queue_edit()
 
    return 0;
 }
+#endif
 
 // below are only non-builtin commands
 #define args	  (parent->args)
 #define exit_code (parent->exit_code)
 #define output	  (parent->output)
 #define session	  (parent->session)
-#define Clone()	  session->Clone()
 #define eprintf	  parent->eprintf
 
 CMD(lcd)
@@ -1174,9 +1223,20 @@ CMD(ls)
 
    FileCopyPeer *src_peer=0;
    if(!nlist)
-      src_peer=new FileCopyPeerDirList(Clone(),args);
+   {
+      FileCopyPeerDirList *dir_list=new FileCopyPeerDirList(session->Clone(),args);
+
+      src_peer=dir_list;
+      ResValue color=ResMgr::Query("color:use-color",0);
+      bool use;
+      if(!strcasecmp(color,"auto"))
+	 use=!output && isatty(1);
+      else
+	 use=color.to_bool();
+      dir_list->UseColor(use);
+   }
    else
-      src_peer=new FileCopyPeerFA(Clone(),a,mode);
+      src_peer=new FileCopyPeerFA(session->Clone(),a,mode);
 
    if(re)
       src_peer->NoCache();
@@ -1205,6 +1265,10 @@ const char *FileSetOutput::parse_argv(ArgV *a)
 {
    static struct option cls_options[] = {
       {"basename",no_argument,0,'B'},
+      {"directory",no_argument,0,'d'},
+      {"human-readable",no_argument,0,'h'},
+      {"block-size",required_argument,0,0},
+      {"si",no_argument,0,0},
       {"classify",no_argument,0,'F'},
       {"long",no_argument,0,'l'},
       {"quiet",no_argument,0,'q'},
@@ -1215,13 +1279,19 @@ const char *FileSetOutput::parse_argv(ArgV *a)
       {"dirsfirst", no_argument,0,'D'},
 
       {"sort", required_argument,0, 0},
+      {"user", no_argument,0, 0},
+      {"group", no_argument,0, 0},
+      {"perms", no_argument,0, 0},
+      {"date", no_argument,0, 0},
+      {"linkcount", no_argument,0, 0},
+      {"links", no_argument,0, 0},
       {0, no_argument, 0, 'S'}, // sort by size
       {0,0,0,0}
    };
 
    a->rewind();
    int opt, longopt;
-   while((opt=a->getopt_long(":1BFilqsDIS", cls_options, &longopt))!=EOF)
+   while((opt=a->getopt_long(":1BdFhiklqsDIS", cls_options, &longopt))!=EOF)
    {
       switch(opt) {
       case 0:
@@ -1231,6 +1301,24 @@ const char *FileSetOutput::parse_argv(ArgV *a)
 	    else return _("invalid argument for `--sort'");
 	 } else if(!strcmp(cls_options[longopt].name, "filesize")) {
 	    size_filesonly = true;
+	 } else if(!strcmp(cls_options[longopt].name, "user")) {
+	    mode |= USER;
+	 } else if(!strcmp(cls_options[longopt].name, "group")) {
+	    mode |= GROUP;
+	 } else if(!strcmp(cls_options[longopt].name, "perms")) {
+	    mode |= PERMS;
+	 } else if(!strcmp(cls_options[longopt].name, "date")) {
+	    mode |= DATE;
+	 } else if(!strcmp(cls_options[longopt].name, "linkcount")) {
+	    mode |= NLINKS;
+	 } else if(!strcmp(cls_options[longopt].name, "links")) {
+	    mode |= LINKS;
+	 } else if(!strcmp(cls_options[longopt].name, "si")) {
+	    output_block_size = -1000;
+	 } else if(!strcmp(cls_options[longopt].name, "block-size")) {
+	    if(!isdigit(optarg[0]))
+	       return _("invalid block size");
+	    output_block_size = atoi(optarg);
 	 }
 	 break;
       case('1'):
@@ -1239,12 +1327,21 @@ const char *FileSetOutput::parse_argv(ArgV *a)
       case('B'):
 	 basenames = true;
          break;
+      case('d'):
+	 list_directories = true;
+         break;
+      case('h'):
+	 output_block_size = -1024;
+         break;
       case('l'):
 	 long_list();
          break;
       case('i'):
 	 patterns_casefold = true;
          break;
+      case('k'):
+	 output_block_size = 1024;
+         break;
       case('F'):
          classify=true;
          break;
@@ -1295,20 +1392,19 @@ CMD(cls)
    if(!strncmp(op,"re",2))
       re=true;
 
+   ArgV arg("", ResMgr::Query("cmd:cls-default", 0));
+   fso.parse_argv(&arg);
+
    if(const char *err = fso.parse_argv(args)) {
-      if(strcmp(err, "ERR"))
-	      eprintf(_("%s: %s.\n"), op, err);
+      eprintf("%s: %s.\n", op, err);
       eprintf(_("Try `help %s' for more information.\n"),op);
       return 0;
    }
 
-   ArgV arg("", ResMgr::Query("cmd:cls-default", 0));
-   fso.parse_argv(&arg);
-
    char *a=args->Combine(0);
 
    FileCopyPeer *src_peer=0;
-   src_peer=new FileCopyPeerCLS(Clone(),args,fso);
+   src_peer=new FileCopyPeerCLS(session->Clone(),args,fso);
    args=0;
 
    if(re)
@@ -1366,7 +1462,7 @@ CMD(cat)
       eprintf(_("Usage: %s [OPTS] files...\n"),op);
       return 0;
    }
-   CatJob *j=new CatJob(Clone(),output,args);
+   CatJob *j=new CatJob(session->Clone(),output,args);
    if(!auto_ascii)
    {
       if(ascii)
@@ -1383,11 +1479,12 @@ CMD(get)
 {
    int opt;
    bool cont=false;
-   const char *opts="+cEuaO:";
+   const char *opts="+cEeuaO:";
    const char *op=args->a0();
    ArgV	 *get_args=new ArgV(op);
    int n_conn=0;
    bool del=false;
+   bool del_target=false;
    bool ascii=false;
    bool glob=false;
    bool make_dirs=false;
@@ -1412,12 +1509,12 @@ CMD(get)
    else if(!strcmp(op,"mget"))
    {
       glob=true;
-      opts="cEadO:";
+      opts="cEeadO:";
    }
    else if(!strcmp(op,"mput"))
    {
       glob=true;
-      opts="cEadO:";
+      opts="cEeadO:";
       reverse=true;
    }
    while((opt=args->getopt(opts))!=EOF)
@@ -1438,6 +1535,9 @@ CMD(get)
       case('E'):
 	 del=true;
 	 break;
+      case('e'):
+	 del_target=true;
+	 break;
       case('a'):
 	 ascii=true;
 	 break;
@@ -1468,7 +1568,7 @@ CMD(get)
       // remove options
       while(args->getindex()>1)
 	 args->delarg(1);
-      mgetJob *j=new mgetJob(Clone(),args,cont,make_dirs);
+      mgetJob *j=new mgetJob(session->Clone(),args,cont,make_dirs);
       if(reverse)
 	 j->Reverse();
       if(del)
@@ -1503,9 +1603,11 @@ CMD(get)
 
    if(n_conn==0)
    {
-      GetJob *j=new GetJob(Clone(),get_args,cont);
+      GetJob *j=new GetJob(session->Clone(),get_args,cont);
       if(del)
 	 j->DeleteFiles();
+      if(del_target)
+	 j->RemoveTargetFirst();
       if(ascii)
 	 j->Ascii();
       if(reverse)
@@ -1514,7 +1616,7 @@ CMD(get)
    }
    else
    {
-      pgetJob *j=new pgetJob(Clone(),get_args);
+      pgetJob *j=new pgetJob(session->Clone(),get_args);
       if(n_conn!=-1)
 	 j->SetMaxConn(n_conn);
       return j;
@@ -1571,29 +1673,18 @@ CMD(rm)
 	 return 0;
       }
    }
-   args->back();
-   char *curr=args->getnext();
-   if(curr==0)
+
+   if(args->getcurr()==0)
       goto print_usage;
 
-   if(recursive)
-   {
-      FinderJob *j=new FinderJob_Cmd(Clone(),args,FinderJob_Cmd::RM);
-      args=0;
-      if(silent)
-	 j->BeQuiet();
-      return j;
-   }
-
    rmJob *j=(rmdir?
-	     new rmdirJob(Clone(),new ArgV(args->a0())):
-	     new rmJob(Clone(),new ArgV(args->a0())));
+	     new rmdirJob(session->Clone(),args):
+	     new rmJob(session->Clone(),args));
 
-   while(curr)
-   {
-      j->AddFile(curr);
-      curr=args->getnext();
-   }
+   if(recursive)
+      j->Recurse();
+
+   args=0;
 
    if(silent)
       j->BeQuiet();
@@ -1602,7 +1693,7 @@ CMD(rm)
 }
 CMD(mkdir)
 {
-   Job *j=new mkdirJob(Clone(),args);
+   Job *j=new mkdirJob(session->Clone(),args);
    args=0;
    return j;
 }
@@ -1855,7 +1946,7 @@ CMD(kill)
 #endif
       return 0;
    }
-   if(!strcmp(args->getarg(1),"all"))
+   if(!strcasecmp(args->getarg(1),"all"))
    {
       parent->KillAll();
       exit_code=0;
@@ -1991,7 +2082,7 @@ CMD(wait)
    char *jn=args->getnext();
    if(jn)
    {
-      if(!strcmp(jn,"all"))
+      if(!strcasecmp(jn,"all"))
       {
 	 parent->WaitForAllChildren();
 	 for(int i=0; i<parent->waiting_num; i++)
@@ -2027,6 +2118,11 @@ CMD(wait)
       eprintf(_("%s: some other job waits for job %d\n"),op,n);
       return 0;
    }
+   if(j->Job::CheckForWaitLoop(parent))
+   {
+      eprintf(_("%s: wait loop detected\n"),op);
+      return 0;
+   }
    j->parent=0;
    j->Bg();
    return j;
@@ -2034,7 +2130,7 @@ CMD(wait)
 
 CMD(subsh)
 {
-   CmdExec *e=new CmdExec(Clone());
+   CmdExec *e=new CmdExec(session->Clone(),parent->cwd->Clone());
 
    char *c=args->getarg(1);
    e->FeedCmd(c);
@@ -2052,7 +2148,7 @@ CMD(mv)
       eprintf(_("Usage: mv <file1> <file2>\n"));
       return 0;
    }
-   Job *j=new mvJob(Clone(),args->getarg(1),args->getarg(2));
+   Job *j=new mvJob(session->Clone(),args->getarg(1),args->getarg(2));
    return j;
 }
 
@@ -2084,15 +2180,15 @@ CMD(cache)  // cache control
    }
 
    exit_code=0;
-   if(!op || !strcmp(op,"status"))
+   if(!op || !strcasecmp(op,"status"))
       LsCache::List();
-   else if(!strcmp(op,"flush"))
+   else if(!strcasecmp(op,"flush"))
       LsCache::Flush();
-   else if(!strcmp(op,"on"))
+   else if(!strcasecmp(op,"on"))
       LsCache::On();
-   else if(!strcmp(op,"off"))
+   else if(!strcasecmp(op,"off"))
       LsCache::Off();
-   else if(!strcmp(op,"size"))
+   else if(!strcasecmp(op,"size"))
    {
       op=args->getnext();
       if(!op)
@@ -2102,7 +2198,7 @@ CMD(cache)  // cache control
 	 return 0;
       }
       long lim=-1;
-      if(strcmp(op,"unlim") && sscanf(op,"%ld",&lim)!=1)
+      if(strcasecmp(op,"unlim") && sscanf(op,"%ld",&lim)!=1)
       {
 	 eprintf(_("%s: Invalid number for size\n"),args->a0());
 	 exit_code=1;
@@ -2110,7 +2206,7 @@ CMD(cache)  // cache control
       }
       LsCache::SetSizeLimit(lim);
    }
-   else if(!strcmp(op,"expire"))
+   else if(!strcasecmp(op,"expire"))
    {
       op=args->getnext();
       if(!op)
@@ -2282,7 +2378,6 @@ static ResDecl res_save_passwords
 
 CMD(bookmark)
 {
-
    args->rewind();
    const char *op=args->getnext();
 
@@ -2303,15 +2398,15 @@ CMD(bookmark)
       return 0;
    }
 
-   if(!strcmp(op,"list"))
+   if(!strcasecmp(op,"list"))
    {
-      char *list=lftp_bookmarks.Format();
+      char *list=lftp_bookmarks.FormatHidePasswords();
       Job *j=CopyJob::NewEcho(list,output,args->a0());
       xfree(list);
       output=0;
       return j;
    }
-   else if(!strcmp(op,"add"))
+   else if(!strcasecmp(op,"add"))
    {
       const char *key=args->getnext();
       if(key==0 || key[0]==0)
@@ -2343,7 +2438,7 @@ CMD(bookmark)
    	 exit_code=0;
       }
    }
-   else if(!strcmp(op,"delete"))
+   else if(!strcasecmp(op,"delete"))
    {
       const char *key=args->getnext();
       if(key==0 || key[0]==0)
@@ -2356,11 +2451,11 @@ CMD(bookmark)
 	 exit_code=0;
       }
    }
-   else if(!strcmp(op,"edit"))
+   else if(!strcasecmp(op,"edit"))
    {
       parent->PrependCmd("shell \"/bin/sh -c 'exec ${EDITOR:-vi} $HOME/.lftp/bookmarks'\"\n");
    }
-   else if(!strcmp(op,"import"))
+   else if(!strcasecmp(op,"import"))
    {
       op=args->getnext();
       if(!op)
@@ -2414,12 +2509,12 @@ CMD(find)
       {"maxdepth",required_argument,0,'d'},
       {0,0,0,0}
    };
-   int opt;
+   int opt, longopt;
    int maxdepth = -1;
    const char *op=args->a0();
 
    args->rewind();
-   while((opt=args->getopt_long("+d:",find_options,0))!=EOF)
+   while((opt=args->getopt_long("+d:",find_options,&longopt))!=EOF)
    {
       switch(opt)
       {
@@ -2439,7 +2534,7 @@ CMD(find)
 
    if(!args->getcurr())
       args->Append(".");
-   FinderJob_List *j=new class FinderJob_List(Clone(),args,
+   FinderJob_List *j=new class FinderJob_List(session->Clone(),args,
       output?output:new FDStream(1,"<stdout>"));
    j->set_maxdepth(maxdepth);
    output=0;
@@ -2458,6 +2553,7 @@ CMD(du)
       {"maxdepth",required_argument,0,'d'},
       {"total",no_argument,0,'c'},
       {"max-depth",required_argument,0,'d'},
+      {"files",no_argument,0,'F'},
       {"human-readable",no_argument,0,'h'},
       {"si",no_argument,0,'H'},
       {"kilobytes",required_argument,0,'k'},
@@ -2474,6 +2570,7 @@ CMD(du)
    bool summarize_only = false;
    bool print_totals=false;
    bool all_files=false;
+   bool file_count=false;
    const char *exclude=0;
 
    exit_code=1;
@@ -2482,7 +2579,7 @@ CMD(du)
 
    args->rewind();
    int opt, longopt;
-   while((opt=args->getopt_long("+abcd:hHkmsS",du_options,&longopt))!=EOF)
+   while((opt=args->getopt_long("+abcd:FhHkmsS",du_options,&longopt))!=EOF)
    {
       switch(opt)
       {
@@ -2504,6 +2601,9 @@ CMD(du)
 	 maxdepth = atoi(optarg);
 	 max_depth_specified = true;
 	 break;
+      case 'F':
+	 file_count=true;
+	 break;
       case 'h':
 	 blocksize = -1024;
 	 break;
@@ -2529,7 +2629,7 @@ CMD(du)
 	       eprintf(_("%s: invalid block size `%s'\n"),op,optarg);
 	       return 0;
 	    }
-	    
+
 	    blocksize = atoi(optarg);
 	    break;
 	 }
@@ -2540,6 +2640,7 @@ CMD(du)
 	 }
 	 /* fallthrough */
       case '?':
+      default:
 	 eprintf(_("Usage: %s [options] <dirs>\n"),op);
 	 return 0;
       }
@@ -2554,6 +2655,15 @@ CMD(du)
       return 0;
    }
 
+   /* It doesn't really make sense to show all files when doing a file count.
+    * We might have -a in an alias as defaults, so let's just silently turn
+    * it off.  (I'm not sure if we should do this for summarize_only and
+    * max_depth_specified, too.) */
+   if (file_count && all_files)
+      all_files=false;
+   if (file_count)
+      blocksize=1;
+
    exit_code=0;
 
    if (summarize_only)
@@ -2561,7 +2671,7 @@ CMD(du)
 
    if(!args->getcurr())
       args->Append(".");
-   FinderJob_Du *j=new class FinderJob_Du(Clone(),args,
+   FinderJob_Du *j=new class FinderJob_Du(session->Clone(),args,
       output?output:new FDStream(1,"<stdout>"));
    args=0;
    j->PrintDepth(maxdepth);
@@ -2572,12 +2682,18 @@ CMD(du)
       j->AllFiles();
    if(separate_dirs)
       j->SeparateDirs();
+   if(file_count)
+      j->FileCount();
    /* if separate_dirs is on, then there's no point in traversing past
     * max_print_depth at all */
    if(separate_dirs && maxdepth != -1)
       j->set_maxdepth(maxdepth);
    if(exclude)
-      j->SetExclude(exclude);
+   {
+      PatternSet *p=new PatternSet();
+      p->Add(p->EXCLUDE,new PatternSet::Glob(exclude));
+      j->SetExclude(p);
+   }
 
    output=0;
    return j;
@@ -2620,9 +2736,12 @@ CMD(lpwd)
       eprintf("%s: %s\n",args->a0(),_("cannot get current directory"));
       return 0;
    }
-   printf("%s\n",parent->cwd);
-   exit_code=0;
-   return 0;
+   const char *name=parent->cwd->GetName();
+   char *buf=alloca_strdup2(name,2);
+   sprintf(buf,"%s\n",name?name:"?");
+   Job *j=CopyJob::NewEcho(buf,strlen(buf),output,args->a0());
+   output=0;
+   return j;
 }
 
 CMD(glob)
@@ -2632,19 +2751,79 @@ CMD(glob)
 
 CMD(chmod)
 {
-   if(args->count()<3)
+   ChmodJob::verbosity verbose = ChmodJob::V_NONE;
+   bool recurse = false, quiet = false;
+
+   static struct option chmod_options[]=
    {
-      eprintf(_("Usage: %s mode file...\n"),args->a0());
+      {"verbose",no_argument,0,'v'},
+      {"changes",no_argument,0,'c'},
+      {"recursive",no_argument,0,'R'},
+      {"silent",no_argument,0,'f'},
+      {"quiet",no_argument,0,'f'},
+      {0,0,0,0}
+   };
+   int opt;
+   int modeind = 0;
+
+   while((opt=args->getopt_long("vcRfrwxXstugoa,+-=",chmod_options,0))!=EOF)
+   {
+      switch(opt)
+      {
+      case 'r': case 'w': case 'x':
+      case 'X': case 's': case 't':
+      case 'u': case 'g': case 'o':
+      case 'a':
+      case ',':
+      case '+': case '=':
+	 modeind = optind?optind-1:1;
+	 break; /* mode string that begins with - */
+
+      case 'v':
+	 verbose=ChmodJob::V_ALL;
+	 break;
+      case 'c':
+	 verbose=ChmodJob::V_CHANGES;
+	 break;
+      case 'R':
+	 recurse = true;
+	 break;
+      case 'f':
+	 quiet = true;
+	 break;
+
+      case '?':
+      usage:
+	 eprintf(_("Usage: %s [OPTS] mode file...\n"),args->a0());
+	 return 0;
+      }
+   }
+
+   if(modeind == 0)
+      modeind = args->getindex();
+
+   char *arg = alloca_strdup(args->getarg(modeind));
+   if(!arg)
+      goto usage;
+   args->delarg(modeind);
+
+   if(!args->getcurr())
+      goto usage;
+
+   mode_change *m = mode_compile(arg, MODE_MASK_ALL);
+   if(m == MODE_INVALID)
+   {
+      eprintf(_("invalid mode string: %s\n"), arg);
       return 0;
    }
-   int m;
-   if(sscanf(args->getarg(1),"%o",&m)!=1)
-   {
-      eprintf(_("%s: %s - not an octal number\n"),args->a0(),args->getarg(1));
-      return 0;
-   }
-   args->delarg(1);
-   Job *j=new ChmodJob(Clone(),m,args);
+
+   ChmodJob *j=new ChmodJob(session->Clone(),args);
+   j->SetVerbosity(verbose);
+   j->SetMode(m);
+   if(quiet)
+      j->BeQuiet(); /* does not affect messages from Verbosity */
+   if(recurse)
+      j->Recurse();
    args=0;
    return j;
 }
@@ -2659,9 +2838,12 @@ CMD(get1)
    static struct option get1_options[]=
    {
       {"ascii",no_argument,0,'a'},
-      {"region",required_argument,0,'r'},
+      {"source-region",required_argument,0,'r'},
+      {"target-region",required_argument,0,'R'},
       {"continue",no_argument,0,'c'},
       {"output",required_argument,0,'o'},
+      {"remove-source-later",no_argument,0,'E'},
+      {"remove-target-first",no_argument,0,'e'},
       {0,0,0,0}
    };
    int opt;
@@ -2747,7 +2929,7 @@ CMD(get1)
    FileCopyPeer *src_peer=0;
    FileCopyPeer *dst_peer=0;
 
-   src_peer=FileCopyPeerFA::New(Clone(),src,FA::RETRIEVE,true);
+   src_peer=FileCopyPeerFA::New(session->Clone(),src,FA::RETRIEVE,true);
 
    if(dst_url.proto==0)
       dst_peer=FileCopyPeerFDStream::NewPut(dst,cont);
