@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: ftpclass.cc,v 1.213.2.4 2002/01/24 10:35:55 lav Exp $ */
+/* $Id: ftpclass.cc,v 1.240 2002/03/10 10:11:44 lav Exp $ */
 
 #include <config.h>
 
@@ -36,10 +36,11 @@
 #include "xalloca.h"
 #include "url.h"
 #include "FtpListInfo.h"
-#include "FtpGlob.h"
+#include "Glob.h"
 #include "FtpDirList.h"
 #include "log.h"
 #include "FileCopyFtp.h"
+#include "LsCache.h"
 
 #include "ascii_ctype.h"
 #include "misc.h"
@@ -68,6 +69,10 @@ enum {FTP_TYPE_A,FTP_TYPE_I};
 # include <termios.h>
 #endif
 
+CDECL_BEGIN
+#include <regex.h>
+CDECL_END
+
 #if defined(HAVE_INET_ATON) && !defined(HAVE_INET_ATON_DECL)
 CDECL int inet_aton(const char *,struct in_addr *);
 #endif
@@ -95,6 +100,8 @@ const bool Ftp::ftps=false;
 #define is2XX(code) ((code)>=200 && (code)<300)
 #define is1XX(code) ((code)>=100 && (code)<200)
 
+#define debug(a) Log::global->Format a
+
 #ifdef TIOCOUTQ
 static bool TIOCOUTQ_returns_free_space;
 static bool TIOCOUTQ_works;
@@ -290,12 +297,7 @@ void Ftp::NoFileCheck(int act)
    }
    if(is5XX(act))
    {
-      // retry on these errors (ftp server ought to send 4xx code instead)
-      if((strstr(line,"Broken pipe") && (!file || !strstr(file,"Broken pipe")))
-      || (strstr(line,"Too many")    && (!file || !strstr(file,"Too many")))
-      || (strstr(line,"timed out")   && (!file || !strstr(file,"timed out")))
-      // if there were some data received, assume it is temporary error.
-      || (mode!=STORE && (flags&IO_FLAG)))
+      if(Transient5XX(act))
       {
 	 Disconnect();
 	 return;
@@ -317,6 +319,30 @@ void Ftp::NoFileCheck(int act)
    Disconnect();
 }
 
+/* 5xx that aren't errors at all */
+bool Ftp::NonError5XX(int act)
+{
+   /* don't strstr; we might have done "NLST No files found" */
+   return (mode==LIST && act==RESP_NO_FILE && !strcmp(line,"550 No files found."));
+}
+
+/* 5xx that are really transient like 4xx */
+bool Ftp::Transient5XX(int act)
+{
+   if(!is5XX(act))
+      return false;
+
+   // retry on these errors (ftp server ought to send 4xx code instead)
+   if((strstr(line,"Broken pipe") && (!file || !strstr(file,"Broken pipe")))
+   || (strstr(line,"Too many")    && (!file || !strstr(file,"Too many")))
+   || (strstr(line,"timed out")   && (!file || !strstr(file,"timed out")))
+   // if there were some data received, assume it is temporary error.
+   || (mode!=STORE && (flags&IO_FLAG)))
+      return true;
+
+   return false;
+}
+
 // 226 Transfer complete.
 void Ftp::TransferCheck(int act)
 {
@@ -369,13 +395,13 @@ void Ftp::TransferCheck(int act)
 	 real_pos=pos=p;
       return;
    }
-   if(copy_mode!=COPY_NONE && act==425 && strstr(line,"port theft"))
+   if(copy_mode!=COPY_NONE && act==425)
    {
       copy_passive=!copy_passive;
       copy_failed=true;
       return;
    }
-   if(act==RESP_NO_FILE && mode==LIST)
+   if(NonError5XX(act))
    {
       DataClose();
       state=EOF_STATE;
@@ -563,28 +589,18 @@ char *Ftp::ExtractPWD()
    const char *scan=strchr(line,'"');
    if(scan==0)
       return 0;
+   scan++;
+   const char *right_quote=strrchr(scan,'"');
+   if(!right_quote)
+      return 0;
 
    char *store=pwd;
-   scan++;
-   for(;;)
+   while(scan<right_quote)
    {
-      if(*scan==0)
-	 return 0;
-      if(*scan=='"')
-      {
-	 if(scan[1]=='"')
-	 {
-	    *store++='"';
-	    scan++;
-	 }
-	 else if(scan[1]!=0 && scan[1]!=' ') // some ftpd don't encode '"'
-	    *store++=*scan;
-	 else
-	    break;
-      }
-      else
-	 *store++=*scan;
-      scan++;
+      // This is the method of quote encoding.
+      if(*scan=='"' && scan[1]=='"')
+	 scan++;
+      *store++=*scan++;
    }
 
    if(store==pwd)
@@ -612,6 +628,13 @@ char *Ftp::ExtractPWD()
    return xstrdup(pwd);
 }
 
+void Ftp::SendCWD(const char *path,check_case_t c)
+{
+   SendCmd2("CWD",path);
+   AddResp(RESP_CWD_RMD_DELE_OK,c);
+   SetRespPath(path);
+}
+
 static bool InPrivateNetwork(const sockaddr_u *u)
 {
    if(u->sa.sa_family==AF_INET)
@@ -993,10 +1016,13 @@ const char *Ftp::FindHomeAuto()
    return 0;
 }
 
-void  Ftp::GetBetterConnection(int level,int count)
+// returns true if we need to sleep instead of moving to higher level.
+bool Ftp::GetBetterConnection(int level,bool limit_reached)
 {
+   bool need_sleep=false;
+
    if(level==0 && cwd==0)
-      return;
+      return need_sleep;
 
    for(FA *fo=FirstSameSite(); fo!=0; fo=NextSameSite(fo))
    {
@@ -1007,14 +1033,20 @@ void  Ftp::GetBetterConnection(int level,int count)
       if(!SameConnection(o))
 	 continue;
 
+      if(level==0 && xstrcmp(real_cwd,o->real_cwd))
+	 continue;
+
       if(o->data_sock!=-1 || o->state!=EOF_STATE || o->mode!=CLOSED)
       {
+	 /* session is in use; last resort is to takeover an active connection */
 	 if(level<2)
 	    continue;
+	 /* only take over lower priority jobs */
 	 if(!connection_takeover || o->priority>=priority)
 	    continue;
 	 if(o->data_sock!=-1 && o->RespQueueSize()<=1)
 	 {
+	    /* don't take over active connections if they won't be able to resume */
 	    if((o->flags&NOREST_MODE) && o->real_pos>0x1000)
 	       continue;
 	    if(o->QueryBool("web-mode",o->hostname))
@@ -1022,7 +1054,7 @@ void  Ftp::GetBetterConnection(int level,int count)
 	    o->DataAbort();
 	    o->DataClose();
 	    if(o->control_sock==-1)
-	       return; // oops...
+	       return need_sleep; // oops...
 	 }
 	 else
 	 {
@@ -1032,16 +1064,29 @@ void  Ftp::GetBetterConnection(int level,int count)
       }
       else
       {
-	 takeover_time=now;
+	 if(limit_reached)
+	 {
+	    /* wait until job is diff seconds idle before taking it over */
+	    int diff=o->last_priority-priority;
+	    if(diff>0)
+	    {
+	       /* number of seconds the task has been idle */
+	       int have_idle=now-o->idle_start;
+	       if(have_idle<diff)
+	       {
+		  TimeoutS(diff-have_idle);
+		  need_sleep=true;
+		  continue;
+	       }
+	    }
+	 }
       }
 
-      if(level==0 && xstrcmp(real_cwd,o->real_cwd))
-	 continue;
-
       // so borrow the connection
       MoveConnectionHere(o);
-      return;
+      return false;
    }
+   return need_sleep;
 }
 
 void  Ftp::HandleTimeout()
@@ -1103,14 +1148,17 @@ int   Ftp::Do()
    {
       // walk through ftp classes and try to find identical idle ftp session
       // first try "easy" cases of session take-over.
-      int count=CountConnections();
       for(int i=0; i<3; i++)
       {
-	 if(i>=2 && (connection_limit==0 || connection_limit>count))
+	 bool limit_reached=(connection_limit>0
+			    && connection_limit<=CountConnections());
+	 if(i>=2 && !limit_reached)
 	    break;
-	 GetBetterConnection(i,count);
+	 bool need_sleep=GetBetterConnection(i,limit_reached);
 	 if(state!=INITIAL_STATE)
 	    return MOVED;
+	 if(need_sleep)
+	    return m;
       }
 
       if(!ReconnectAllowed())
@@ -1142,12 +1190,8 @@ int   Ftp::Do()
 	 }
 	 sprintf(str,"socket: %s",strerror(errno));
          DebugPrint("**** ",str,0);
-	 if(errno==ENFILE || errno==EMFILE)
-	 {
-	    // file table overflow - it could free sometime
-	    TimeoutS(1);
+	 if(NonFatalError(errno))
 	    return m;
-	 }
 	 sprintf(str,_("cannot create socket of address family %d"),
 			peer_sa.sa.sa_family);
 	 SetError(SEE_ERRNO,str);
@@ -1218,8 +1262,8 @@ int   Ftp::Do()
       AddResp(RESP_READY,CHECK_READY);
 
 #ifdef USE_SSL
-      // ssl for anonymous does not make sense.
-      if(!ftps && !proxy && QueryBool("ssl-allow") && user && pass)
+      if(!ftps && !proxy && QueryBool("ssl-allow",hostname) &&
+		   ((user && pass) || QueryBool("ssl-allow-anonymous",hostname)))
       {
 	 const char *auth=Query("ssl-auth",hostname);
 	 SendCmd2("AUTH",auth);
@@ -1336,14 +1380,11 @@ int   Ftp::Do()
 	 SetError(NO_FILE,_("SITE CHMOD is not supported by this site"));
 	 return MOVED;
       }
-
-      if(takeover_time!=NO_DATE && takeover_time+1-priority>now
-      && connection_limit>0 && connection_limit<=CountConnections()+1)
+      if(mode==CHANGE_MODE && !QueryBool("use-site-chmod"))
       {
-	 TimeoutS(takeover_time+1-priority-now);
-	 goto notimeout_return;
+	 SetError(NO_FILE,_("SITE CHMOD is disabled by ftp:use-site-chmod"));
+	 return MOVED;
       }
-      takeover_time=NO_DATE;
 
       if(home==0 && !RespQueueIsEmpty())
 	 goto usual_return;
@@ -1359,9 +1400,7 @@ int   Ftp::Do()
 	 if((!last_cwd && xstrcmp(cwd,real_cwd))
 	    || (last_cwd && xstrcmp(last_cwd->path,cwd)))
 	 {
-	    SendCmd2("CWD",cwd);
-	    AddResp(RESP_CWD_RMD_DELE_OK,CHECK_CWD_CURR);
-	    SetRespPath(cwd);
+	    SendCWD(cwd,CHECK_CWD_CURR);
 	 }
 	 else if(last_cwd && !xstrcmp(last_cwd->path,cwd))
 	 {
@@ -1387,7 +1426,8 @@ int   Ftp::Do()
       if(copy_mode!=COPY_NONE && !copy_passive && !copy_addr_valid)
 	 goto usual_return;
 
-      if(entity_size!=NO_SIZE && entity_size!=NO_SIZE_YET && entity_size<=pos)
+      if(entity_size!=NO_SIZE && entity_size!=NO_SIZE_YET && entity_size<=pos
+      && mode==RETRIEVE)
       {
 	 eof=true;
 	 goto pre_WAITING_STATE; // simulate eof.
@@ -1498,6 +1538,7 @@ int   Ftp::Do()
 	 real_pos=-1;	// we don't yet know if REST will succeed
 
       flags&=~IO_FLAG;
+      last_priority=priority;
 
       switch((enum open_mode)mode)
       {
@@ -1550,8 +1591,7 @@ int   Ftp::Do()
 	    if(!vms_path && !AbsolutePath(file) && real_cwd
 	    && !strncmp(file,real_cwd,len) && file[len]=='/')
 	       path_to_use=file+len+1;
-	    SendCmd2("CWD",path_to_use);
-	    AddResp(RESP_CWD_RMD_DELE_OK,CHECK_CWD);
+	    SendCWD(path_to_use,CHECK_CWD);
 	    SetRespPath(file);
 	 }
 	 goto pre_WAITING_STATE;
@@ -1621,15 +1661,17 @@ int   Ftp::Do()
       {
 	 if(mode==MAKE_DIR && mkdir_p)
 	 {
-	    char *sl;
-	    sl=strchr(file,'/');
+	    char *sl=strchr(file,'/');
 	    while(sl)
 	    {
 	       if(sl>file)
 	       {
 		  *sl=0;
-		  SendCmd2("MKD",file);
-		  AddResp(0,CHECK_IGNORE);
+		  if(strcmp(file,".") && strcmp(file,".."))
+		  {
+		     SendCmd2("MKD",file);
+		     AddResp(0,CHECK_IGNORE);
+		  }
 		  *sl='/';
 	       }
 	       sl=strchr(sl+1,'/');
@@ -1646,7 +1688,17 @@ int   Ftp::Do()
 	 else if(mode==MAKE_DIR)
 	    AddResp(RESP_PWD_MKD_OK,CHECK_FILE_ACCESS);
 	 else if(mode==QUOTE_CMD)
+	 {
 	    AddResp(0,CHECK_IGNORE,true);
+	    if(!strncasecmp(file,"CWD",3)
+	    || !strncasecmp(file,"CDUP",4)
+	    || !strncasecmp(file,"XCWD",4)
+	    || !strncasecmp(file,"XCUP",4))
+	    {
+	       DebugPrint("---- ","Resetting cwd",9);
+	       set_real_cwd(0);  // we do not know the path now.
+	    }
+	 }
 	 else if(mode==RENAME)
 	    AddResp(350,CHECK_RNFR);
 	 else
@@ -2043,12 +2095,8 @@ notimeout_return:
    return m;
 
 system_error:
-   if(errno==ENFILE || errno==EMFILE)
-   {
-      // file table overflow - it could free sometime
-      TimeoutS(1);
+   if(NonFatalError(errno))
       return m;
-   }
    quit_sent=true;
    Disconnect();
    SetError(SEE_ERRNO,0);
@@ -2154,18 +2202,20 @@ int Ftp::ReplyLogPriority(int code)
       return 3;
    if(code==451 && mode==CLOSED)
       return 4;
-   if(code==550 && mode==ARRAY_INFO
-   && !RespQueueIsEmpty() && RespQueue[RQ_head].check_case==CHECK_MDTM)
-      return 4;
-   if(code==550 && mode==LIST && line && strstr(line,"No files found"))
-      return 4;
-   if(code==550 && mode==CHANGE_DIR)
-      return 4;
+   /* Most 5XXs go to level 4, as it's the job's responsibility to
+    * print fatal errors. Some 5XXs are treated as 4XX's; send those
+    * to level 0. (Maybe they should go to 1; we're going to retry them,
+    * after all. */
+   if(is5XX(code))
+      return Transient5XX(code)? 0:4;
 
-   // Error messages
-   // 221 is the reply to QUIT, but we don't expect it.
-   if(code>=400 || (code==221 && RespQueue[RQ_head].expect!=221))
+   if(is4XX(code))
       return 0;
+
+   // 221 is the reply to QUIT, but we don't expect it.
+   if(code==221 && RespQueue[RQ_head].expect!=221)
+      return 0;
+
    return 4;
 }
 
@@ -2304,7 +2354,7 @@ int  Ftp::ReceiveResp()
 	       res=read(control_sock,resp+resp_size,resp_alloc-resp_size-1);
 	       if(res==-1)
 	       {
-		  if(E_RETRY(errno))
+		  if(NonFatalError(errno))
 		     return m;
 		  if(NotSerious(errno))
 		     DebugPrint("**** ",strerror(errno),0);
@@ -2609,7 +2659,7 @@ int  Ftp::FlushSendQueue(bool all)
 	    return m;
 	 if(res==-1)
 	 {
-	    if(E_RETRY(errno))
+	    if(NonFatalError(errno))
 	       return m;
 	    if(NotSerious(errno) || errno==EPIPE)
 	       DebugPrint("**** ",strerror(errno),0);
@@ -2728,16 +2778,14 @@ void Ftp::SendCmd2(const char *cmd,int v)
 
 int   Ftp::SendEOT()
 {
-   if(mode==STORE)
-   {
-      if(state==DATA_OPEN_STATE)
-      {
-	 DataClose();
-	 state=WAITING_STATE;
-      	 return(OK);
-      }
+   if(mode!=STORE)
+      return(OK); /* nothing to do */
+
+   if(state!=DATA_OPEN_STATE)
       return(DO_AGAIN);
-   }
+
+   DataClose();
+   state=WAITING_STATE;
    return(OK);
 }
 
@@ -2986,7 +3034,7 @@ read_again:
       res=read(data_sock,buf,size);
       if(res==-1)
       {
-	 if(E_RETRY(errno))
+	 if(NonFatalError(errno))
 	    return DO_AGAIN;
 	 if(NotSerious(errno))
 	 {
@@ -3119,7 +3167,7 @@ int   Ftp::Write(const void *buf,int size)
       res=write(data_sock,buf,size);
       if(res==-1)
       {
-	 if(E_RETRY(errno))
+	 if(NonFatalError(errno))
 	    return DO_AGAIN;
 	 if(NotSerious(errno) || errno==EPIPE)
 	 {
@@ -3153,16 +3201,18 @@ int   Ftp::StoreStatus()
    if(mode!=STORE)
       return(OK);
 
-   if(state==WAITING_STATE && RespQueueIsEmpty())
-   {
-      eof=true;
-      return(OK);
-   }
    if(state==DATA_OPEN_STATE)
    {
       // have not send EOT by SendEOT, do it now
       SendEOT();
    }
+
+   if(state==WAITING_STATE && RespQueueIsEmpty())
+   {
+      eof=true;
+      return(OK);
+   }
+
    return(IN_PROGRESS);
 }
 
@@ -3285,7 +3335,7 @@ void Ftp::CheckResp(int act)
    if(act==421 || act==221)  // timeout or something else
    {
       if(RespQueueIsEmpty() || RespQueue[RQ_head].expect!=221)
-	 DebugPrint("**** ",_("remote end closes connection"),3);
+	 DebugPrint("**** ",_("remote end closed connection"),3);
       quit_sent=true;
       Disconnect();
       return;
@@ -3305,6 +3355,7 @@ void Ftp::CheckResp(int act)
    // some servers mess all up
    if(act==331 && exp==220 && !(flags&SYNC_MODE) && RespQueueSize()>1)
    {
+      PopResp();
       DebugPrint("---- ",_("Turning on sync-mode"),2);
       ResMgr::Set("ftp:sync-mode",hostname,"on");
       Disconnect();
@@ -3314,6 +3365,10 @@ void Ftp::CheckResp(int act)
 
    bool match=(act/100==exp/100);
    check_case_t cc=RespQueue[RQ_head].check_case;
+   const char *exp_path=alloca_strdup(RespQueue[RQ_head].path);
+
+   // remove first, so that Disconnect() has accurate picture.
+   PopResp();
 
    switch(cc)
    {
@@ -3356,14 +3411,16 @@ void Ftp::CheckResp(int act)
 	 if(cc==CHECK_CWD)
 	 {
 	    xfree(cwd);
-	    cwd=xstrdup(RespQueue[RQ_head].path);
+	    cwd=xstrdup(exp_path);
 	 }
 	 set_real_cwd(cwd);
+	 LsCache::SetDirectory(this, exp_path, true);
 	 break;
       }
       if(is5XX(act))
       {
 	 SetError(NO_FILE,line);
+	 LsCache::SetDirectory(this, exp_path, false);
 	 break;
       }
       Disconnect();
@@ -3371,7 +3428,7 @@ void Ftp::CheckResp(int act)
 
    case CHECK_CWD_STALE:
       if(is2XX(act))
-	 set_real_cwd(RespQueue[RQ_head].path);
+	 set_real_cwd(exp_path);
       goto ignore;
 
    case CHECK_ABOR:
@@ -3466,6 +3523,7 @@ void Ftp::CheckResp(int act)
 	 }
 	 if(!home)
 	    set_home(home_auto);
+	 LsCache::SetDirectory(this, home, true);
 	 break;
       }
       break;
@@ -3498,7 +3556,7 @@ void Ftp::CheckResp(int act)
 
    case CHECK_TRANSFER_CLOSED:
       if(strstr(line,"ABOR")
-      && RespQueueSize()>=2 && RespQueue[RQ_head+1].check_case==CHECK_ABOR)
+      && RespQueueSize()>=2 && RespQueue[RQ_head].check_case==CHECK_ABOR)
       {
 	 DebugPrint("**** ","server bug: 426 reply missed",1);
 	 PopResp();
@@ -3526,14 +3584,15 @@ void Ftp::CheckResp(int act)
 #endif // USE_SSL
 
    } /* end switch */
-
-   PopResp();
 }
 
 void  Ftp::SetRespPath(const char *p)
 {
    if(RQ_tail>RQ_head)
+   {
+      xfree(RespQueue[RQ_tail-1].path);
       RespQueue[RQ_tail-1].path=xstrdup(p);
+   }
 }
 
 void  Ftp::PopResp()
@@ -3587,17 +3646,25 @@ const char *Ftp::CurrentStatus()
    case(CWD_CWD_WAITING_STATE):
       return(_("Changing remote directory..."));
    case(WAITING_STATE):
-      if(copy_mode!=COPY_NONE)
-      {
-	 if(RespQueueIsEmpty())
-	    return _("Waiting for other copy peer...");
-      }
-      if(mode==STORE && copy_mode==COPY_NONE)
+      if(copy_mode==COPY_SOURCE)
+	 return "";
+      if(copy_mode==COPY_DEST && RespQueueIsEmpty())
+	 return _("Waiting for other copy peer...");
+      if(mode==STORE)
 	 return(_("Waiting for transfer to complete"));
       return(_("Waiting for response..."));
    case(ACCEPTING_STATE):
       return(_("Waiting for data connection..."));
    case(DATA_OPEN_STATE):
+#ifdef USE_SSL
+      if(data_ssl)
+      {
+	 if(mode==STORE)
+	    return(_("Sending data/TLS"));
+         else
+	    return(_("Receiving data/TLS"));
+      }
+#endif
       if(data_sock!=-1)
       {
 	 if(mode==STORE)
@@ -3850,13 +3917,13 @@ void Ftp::SetError(int ec,const char *e)
    super::SetError(ec,e);
 }
 
-ListInfo *Ftp::MakeListInfo()
+ListInfo *Ftp::MakeListInfo(const char *path)
 {
-   return new FtpListInfo(this);
+   return new FtpListInfo(this,path);
 }
 Glob *Ftp::MakeGlob(const char *pattern)
 {
-   return new FtpGlob(this,pattern);
+   return new GenericGlob(this,pattern);
 }
 DirList *Ftp::MakeDirList(ArgV *args)
 {
