@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: resource.cc,v 1.64.2.1 2002/01/03 17:49:35 lav Exp $ */
+/* $Id: resource.cc,v 1.73 2002/02/07 12:26:45 lav Exp $ */
 
 #include <config.h>
 
@@ -186,11 +186,13 @@ static ResDecl
    ResDecl11b("ftp:ssl-force",		  "no",	   ResMgr::BoolValidate,0),
    ResDecl11c("ftp:ssl-protect-data",	  "no",	   ResMgr::BoolValidate,0),
    ResDecl11d("ftp:ssl-auth",		  "TLS",   AuthArgValidate,0),
+   ResDecl11e("ftp:ssl-allow-anonymous",  "no",   ResMgr::BoolValidate,0),
 #endif
    ResDecl12 ("ftp:stat-interval",	  "1",	   ResMgr::UNumberValidate,0),
    ResDecl13 ("ftp:sync-mode",		  "on",    ResMgr::BoolValidate,0),
    ResDecl14 ("ftp:use-abor",		  "yes",   ResMgr::BoolValidate,0),
    ResDecl15 ("ftp:use-fxp",		  "yes",   ResMgr::BoolValidate,0),
+   ResDecl15a("ftp:use-site-chmod",	  "yes",   ResMgr::BoolValidate,0),
    ResDecl16a("ftp:use-site-idle",	  "no",    ResMgr::BoolValidate,0),
    ResDecl16 ("ftp:use-stat",		  "yes",   ResMgr::BoolValidate,0),
    ResDecl17 ("ftp:use-quit",		  "yes",   ResMgr::BoolValidate,0),
@@ -218,7 +220,7 @@ static ResDecl
    ResDecl29a("http:post-content-type",   "application/x-www-form-urlencoded",0,0),
    ResDecl29 ("http:put-method",	  "PUT",   PutOrPost,0),
    ResDecl30 ("http:put-content-type",	  "",	   0,0),
-   ResDecl30b("http:referer",		  ".",	   0,0),
+   ResDecl30b("http:referer",		  "",	   0,0),
 #ifdef USE_SSL
    ResDecl30a("https:proxy",		  "",	   HttpProxyValidate,0),
 #endif
@@ -238,8 +240,10 @@ static ResDecl
    ResDecl43 ("net:timeout",		  "300",   ResMgr::UNumberValidate,0),
    ResDecl44 ("net:connection-limit",	  "0",	   ResMgr::UNumberValidate,0),
    ResDecl45 ("net:connection-takeover",  "yes",   ResMgr::BoolValidate,0),
-   ResDecl46 ("mirror:time-precision",	  "1s",    ResMgr::TimeIntervalValidate,ResMgr::NoClosure),
-   ResDecl47 ("mirror:loose-time-precision","24h", ResMgr::TimeIntervalValidate,ResMgr::NoClosure);
+   ResDecl46 ("mirror:order",		  "*.sfv *.sig *.md5* *.sum * */", 0,ResMgr::NoClosure),
+   ResDecl47 ("mirror:parallel-directories", "yes", ResMgr::BoolValidate,ResMgr::NoClosure),
+   ResDecl48 ("mirror:parallel-transfer-count", "1", ResMgr::UNumberValidate,ResMgr::NoClosure),
+   ResDecl49 ("mirror:exclude-regex",	  "(^|/)(\\.in\\.|\\.nfs)",ResMgr::ERegExpValidate,ResMgr::NoClosure);
 
 #if INET6
 # define DEFAULT_ORDER "inet inet6"
@@ -257,10 +261,24 @@ static ResDecl
    res_use_fork	   ("dns:use-fork",     "yes", ResMgr::BoolValidate,ResMgr::NoClosure);
 
 static ResDecl
-   fish_shell  ("fish:shell", "/bin/sh",  0,0);
+   fish_shell  ("fish:shell", "/bin/sh",  0,0),
+   fish_connect("fish:connect-program", "ssh", 0,0);
+
+static ResDecl
+   res_dir_colors ("color:dir-colors",   "",   0,ResMgr::NoClosure);
 
 void ResMgr::ClassInit()
 {
+   for(ResDecl *scan=type_chain; scan; scan=scan->next)
+   {
+      if(scan->defvalue && scan->val_valid)
+      {
+	 const char *error=(*scan->val_valid)(&scan->defvalue);
+	 if(error)
+	    fprintf(stderr,"Default value for %s is invalid: %s\n",scan->name,error);
+      }
+   }
+
    // inherit http proxy from environment
    const char *http_proxy=getenv("http_proxy");
    if(http_proxy)
@@ -306,4 +324,10 @@ void ResMgr::ClassInit()
    const char *module_path=getenv("LFTP_MODULE_PATH");
    if(module_path)
       Set("module:path",0,module_path);
+
+   const char *dir_colors=getenv("LS_COLORS");
+   if(!dir_colors)
+      dir_colors=getenv("ZLS_COLORS"); /* zsh */
+   if(dir_colors)
+      Set("color:dir-colors",0,dir_colors);
 }
